#include "global.h"

static void copy_scrap();               
static void print_scrap_numbers();      
static void format_entry();             
static void format_user_entry();
void write_latex(file_name, tex_name)
     char *file_name;
     char *tex_name;
{
  FILE *tex_file = fopen(tex_name, "w");
  if (tex_file) {
    if (verbose_flag)
      fprintf(stderr, "writing %s\n", tex_name);
    source_open(file_name);
    {
      int scraps = 1;
      int c = source_get();
      while (c != EOF) {
        if (c == '@')
          {
            int big_definition = FALSE;
            c = source_get();
            switch (c) {
              case 'O': big_definition = TRUE;
              case 'o': {
                          Name *name = collect_file_name();
                          {
                            fputs("\\begin{flushleft} \\small", tex_file);
                            if (!big_definition)
                              fputs("\n\\begin{minipage}{\\linewidth}", tex_file);
                            fprintf(tex_file, " \\label{scrap%d}\n", scraps);
                          }
                          fprintf(tex_file, "\\verb@\"%s\"@ {\\footnotesize ", name->spelling);
                          write_single_scrap_ref(tex_file, scraps++);
                          fputs(" }$\\equiv$\n", tex_file);
                          
                          {
                            int c = source_get();
                            LATEX("\\vspace{-1ex}\n");
                            LATEX("\\begin{list}{}{\\advance\\textwidth by -\\leftmargin}");
                            LATEX("\\item\n");
                            if (c == '\n') {TEX("\\par%\n"); c = source_get();}
                            copy_scrap(c,tex_file);
                            LATEX("$\\diamond$\n\\end{list}\n");
                            TEX("$\\diamond$\\par\\endgroup%\n");
                          }
                          
                          
                          {
                            if (name->defs->next) {
                              LATEX("\\vspace{-1ex}\n");
                              LATEX("\\footnotesize\\addtolength{\\baselineskip}{-1ex}\n");
                              LATEX("\\begin{list}{}{\\setlength{\\itemsep}{-\\parsep}");
                              LATEX("\\setlength{\\itemindent}{-\\leftmargin}}\n");
                              LATEX("\\item File defined by scraps ");
                              TEX("\\begingroup\\parskip=0pt\\footnotesize");
                              TEX("\\advance\\leftskip by \\refsmargin%\n");
                              TEX("\\par File defined by scraps ");
                              print_scrap_numbers(tex_file, name->defs);
                              LATEX("\\end{list}\n");
                              TEX("\\par\\endgroup");
                            } else {
                              LATEX("\\vspace{-2ex}\n");
                            }
                          }
                          
                          {
                            if (!big_definition)
                              fputs("\\end{minipage}\\\\[4ex]\n", tex_file);
                            fputs("\\end{flushleft}\n", tex_file);
                            do
                              c = source_get();
                            while (isspace(c));
                          }
                        }
                        break;
              case 'D': big_definition = TRUE;
              case 'd': {
                          Name *name = collect_macro_name();
                          {
                            fputs("\\begin{flushleft} \\small", tex_file);
                            if (!big_definition)
                              fputs("\n\\begin{minipage}{\\linewidth}", tex_file);
                            fprintf(tex_file, " \\label{scrap%d}\n", scraps);
                          }
                          fprintf(tex_file, "$\\langle$%s {\\footnotesize ", name->spelling);
                          write_single_scrap_ref(tex_file, scraps++);
                          fputs("}$\\rangle\\equiv$\n", tex_file);
                          
                          {
                            int c = source_get();
                            LATEX("\\vspace{-1ex}\n");
                            LATEX("\\begin{list}{}{\\advance\\textwidth by -\\leftmargin}");
                            LATEX("\\item\n");
                            if (c == '\n') {TEX("\\par%\n"); c = source_get();}
                            copy_scrap(c,tex_file);
                            LATEX("$\\diamond$\n\\end{list}\n");
                            TEX("$\\diamond$\\par\\endgroup%\n");
                          }
                          
                          {
                            fputs("\\vspace{-1ex}\n", tex_file);
                            fputs("\\footnotesize\\addtolength{\\baselineskip}{-1ex}\n", tex_file);
                            fputs("\\begin{list}{}{\\setlength{\\itemsep}{-\\parsep}", tex_file);
                            fputs("\\setlength{\\itemindent}{-\\leftmargin}}\n", tex_file);
                            if (name->defs->next) {
                              fputs("\\item Macro defined by scraps ", tex_file);
                              print_scrap_numbers(tex_file, name->defs);
                            }
                          }
                          {
                            if (name->uses) {
                              if (name->uses->next) {
                                fputs("\\item Macro referenced in scraps ", tex_file);
                                print_scrap_numbers(tex_file, name->uses);
                              }
                              else {
                                fputs("\\item Macro referenced in scrap ", tex_file);
                                write_single_scrap_ref(tex_file, name->uses->scrap);
                                fputs(".\n", tex_file);
                              }
                            }
                            else {
                              fputs("\\item Macro never referenced.\n", tex_file);
                              fprintf(stderr, "%s: <%s> never referenced.\n",
                                      command_name, name->spelling);
                            }
                            fputs("\\end{list}\n", tex_file);
                          }
                          {
                            if (!big_definition)
                              fputs("\\end{minipage}\\\\[4ex]\n", tex_file);
                            fputs("\\end{flushleft}\n", tex_file);
                            do
                              c = source_get();
                            while (isspace(c));
                          }
                        }
                        break;
              case 'f': {
                          if (file_names) {
                            fputs("\n{\\small\\begin{list}{}{\\setlength{\\itemsep}{-\\parsep}",
                                  tex_file);
                            fputs("\\setlength{\\itemindent}{-\\leftmargin}}\n", tex_file);
                            format_entry(file_names, tex_file, TRUE);
                            fputs("\\end{list}}", tex_file);
                          }
                          c = source_get();
                        }
                        break;
              case 'm': {
                          if (macro_names) {
                            fputs("\n{\\small\\begin{list}{}{\\setlength{\\itemsep}{-\\parsep}",
                                  tex_file);
                            fputs("\\setlength{\\itemindent}{-\\leftmargin}}\n", tex_file);
                            format_entry(macro_names, tex_file, FALSE);
                            fputs("\\end{list}}", tex_file);
                          }
                          c = source_get();
                        }
                        break;
              case 'u': {
                          if (user_names) {
                            fputs("\n{\\small\\begin{list}{}{\\setlength{\\itemsep}{-\\parsep}",
                                  tex_file);
                            fputs("\\setlength{\\itemindent}{-\\leftmargin}}\n", tex_file);
                            format_user_entry(user_names, tex_file);
                            fputs("\\end{list}}", tex_file);
                          }
                          c = source_get();
                        }
                        break;
              case '@': putc(c, tex_file);
              default:  c = source_get();
                        break;
            }
          }
        else {
          putc(c, tex_file);
          c = source_get();
        }
      }
    }
    fclose(tex_file);
  }
  else
    fprintf(stderr, "%s: can't open %s\n", command_name, tex_name);
}
static void print_scrap_numbers(tex_file, scraps)
     FILE *tex_file;
     Scrap_Node *scraps;
{
  int page;
  write_scrap_ref(tex_file, scraps->scrap, TRUE, &page);
  scraps = scraps->next;
  while (scraps) {
    write_scrap_ref(tex_file, scraps->scrap, FALSE, &page);
    scraps = scraps->next;
  }
  fputs(".\n", tex_file);
}
static void copy_scrap(c,file)
    char c;
    FILE *file;
{
  int indent = 0; 
  int codestate = 0;
  int verbmode=0;
  while (1) {
    switch (c) {
      case '@':  {
                   c = source_get();
                   switch (c) {
                     case '@': if (verbmode) fputs("@{\\tt @}\\verb@", file);
                               else fputs("{\\tt @}", file);
                               break;
                     case '|': if (codestate==1) {
                                 codestate=2;
                                 {
                                 if (verbmode) putc('@',file);
                                 fputs("}\\setbox1\\vtop{\\hsize=0.4\\textwidth\\leftskip=10pt\n",file);
                                 verbmode=0;
                                 }
                               } else {
                                 if (verbmode) putc('@',file);
                                 {
                                   do {
                                     do
                                       c = source_get();
                                     while (c != '@');
                                     c = source_get();
                                   } while (c != '}');
                                 }
                                 return;
                               }
                               break;
                     case '}': if (codestate==0) {
                                 if (verbmode) putc('@', file); return;
                               } else if (codestate==1) {
                                 if (verbmode) putc('@', file); codestate=0; 
                                 break;
                               } else if (codestate==2) {
                                 {
                                 codestate=0; verbmode=0;
                                 fputs("}\\ifnum\\dp1>\\dp0\\dp0=\\dp1\\else\\dp1=\\dp0\\fi\n",file);
                                 fputs("\\hbox{\\box0\\vrule\\box1}\n",file);
                                 fputs("\\endgroup",file);
                                 if (source_peek=='\n') c = source_get();
                                 }
                               }
                               break;
                     case '{': if (codestate==0) {
                                                  {
                                                    codestate=1;
                                                    if (verbmode) putc('@',file);
                                                    fputs("\\begingroup\\catcode`\\^^M=5%\n",file);
                                                    fputs("\\setbox0\\vtop{\\hsize=0.6\\textwidth\\raggedright%\n",file);
                                                  
                                                    if (source_peek=='\n') c = source_get();
                                                  
                                                  }
                                                  }
                               break;
                     case '<': {
                                 Name *name = collect_scrap_name();
                                 if (verbmode) putc('@',file);
                                 fprintf(file, "$\\langle$%s {\\footnotesize ", name->spelling);
                                 if (name->defs)
                                   {
                                     Scrap_Node *p = name->defs;
                                     write_single_scrap_ref(file, p->scrap);
                                     p = p->next;
                                     if (p)
                                       fputs(", \\ldots\\ ", file);
                                   }
                                 else {
                                   putc('?', file);
                                   fprintf(stderr, "%s: scrap never defined <%s>\n",
                                           command_name, name->spelling);
                                 }
                                 fputs("}$\\rangle$", file);
                                 if (verbmode) fputs("\\verb@",file);
                               }
                               break;
                     default:  /* ignore these since pass1 will have warned about them */
                               break;
                   }
                 }
                 break;
      case '\n': 
                 if (codestate!=2 && !verbmode) {
                   verbmode=1; fputs("\\mbox{}\\verb@", file);
                 }
                 if (codestate!=2) {
                   verbmode=0; fputs("@\\\\\n", file); indent = 0;
                 } else if (codestate==2) fputs("\n", file);
                 break;
      case '\t': 
                 if (codestate!=2 && !verbmode) {
                   verbmode=1; fputs("\\mbox{}\\verb@", file);
                 }
                 {
                   int delta = 8 - (indent % 8);
                   indent += delta;
                   while (delta > 0) {
                     putc(' ', file);
                     delta--;
                   }
                 }
                 break;
      default:
                 if (codestate!=2 && !verbmode) {
                   verbmode=1; fputs("\\mbox{}\\verb@", file);
                 }
                 if (cross_flag && isalpha(c)) {{
                                                Scrap_Node *defs;
                                                char spelling[100];
                                                char *sp = spelling;
                                                int first;
                                                int page; page=-2;
                                                while (1) {
                                                  *sp++ = c; putc(c,file); indent++;
                                                  if (!(isalpha(source_peek) || isdigit(source_peek) || source_peek=='_'))
                                                    break;
                                                  c = source_get();
                                                }
                                                *sp = '\0';
                                                defs = get_defs(spelling, user_names);
                                                if (defs) {
                                                  if (verbmode) putc('@',file);
                                                  fputs("$_{",file);
                                                  first=1;
                                                  while (defs) {
                                                    write_scrap_ref(file,defs->scrap,first,&page);
                                                    first = 0;
                                                    defs = defs->next;
                                                  }
                                                  
                                                  fputs("}$",file);
                                                  if (verbmode) fputs("\\verb@",file);
                                                }
                                                }
                                                }
                 else {putc(c, file); indent++; break;}
    }
    c = source_get();
  }
}
static void format_entry(name, tex_file, file_flag)
     Name *name;
     FILE *tex_file;
     int file_flag;
{
  while (name) {
    format_entry(name->llink, tex_file, file_flag);
    {
      fputs("\\item ", tex_file);
      if (file_flag) {
        fprintf(tex_file, "\\verb@\"%s\"@ ", name->spelling);
        {
          Scrap_Node *p = name->defs;
          fputs("{\\footnotesize Defined by scrap", tex_file);
          if (p->next) {
            fputs("s ", tex_file);
            print_scrap_numbers(tex_file, p);
          }
          else {
            putc(' ', tex_file);
            write_single_scrap_ref(tex_file, p->scrap);
            putc('.', tex_file);
          }
          putc('}', tex_file);
        }
      }
      else {
        fprintf(tex_file, "$\\langle$%s {\\footnotesize ", name->spelling);
        {
          Scrap_Node *p = name->defs;
          if (p) {
            int page;
            write_scrap_ref(tex_file, p->scrap, TRUE, &page);
            p = p->next;
            while (p) {
              write_scrap_ref(tex_file, p->scrap, FALSE, &page);
              p = p->next;
            }
          }
          else
            putc('?', tex_file);
        }
        fputs("}$\\rangle$ ", tex_file);
        {
          Scrap_Node *p = name->uses;
          fputs("{\\footnotesize ", tex_file);
          if (p) {
            fputs("Referenced in scrap", tex_file);
            if (p->next) {
              fputs("s ", tex_file);
              print_scrap_numbers(tex_file, p);
            }
            else {
              putc(' ', tex_file);
              write_single_scrap_ref(tex_file, p->scrap);
              putc('.', tex_file);
            }
          }
          else
            fputs("Not referenced.", tex_file);
          putc('}', tex_file);
        }
      }
      putc('\n', tex_file);
    }
    name = name->rlink;
  }
}
static void format_user_entry(name, tex_file)
     Name *name;
     FILE *tex_file;
{
  while (name) {
    format_user_entry(name->llink, tex_file);
    {
      Scrap_Node *uses = name->uses;
      if (uses) {
        int page;
        Scrap_Node *defs = name->defs;
        fprintf(tex_file, "\\item \\verb@%s@: ", name->spelling);
        if (uses->scrap < defs->scrap) {
          write_scrap_ref(tex_file, uses->scrap, TRUE, &page);
          uses = uses->next;
        }
        else {
          if (defs->scrap == uses->scrap)
            uses = uses->next;
          fputs("\\underline{", tex_file);
          write_single_scrap_ref(tex_file, defs->scrap);
          putc('}', tex_file);
          page = -2;
          defs = defs->next;
        }
        while (uses || defs) {
          if (uses && (!defs || uses->scrap < defs->scrap)) {
            write_scrap_ref(tex_file, uses->scrap, FALSE, &page);
            uses = uses->next;
          }
          else {
            if (uses && defs->scrap == uses->scrap)
              uses = uses->next;
            fputs(", \\underline{", tex_file);
            write_single_scrap_ref(tex_file, defs->scrap);
            putc('}', tex_file);
            page = -2;
            defs = defs->next;
          }
        }
        fputs(".\n", tex_file);
      }
    }
    name = name->rlink;
  }
}
