\input nutmacs
\input mytexmacros
\newif\ifshowcode
\showcodetrue

\begingroup
\input titlepage
\footline={\hfil}
\titlebox{\Large\bf Nutweb Version 1.5\\
\vskip10pt
\rm A Simple Literate Programming Tool\\
\vskip6pt
\normal\rm Preston Briggs$^\dagger$\\ 
\tt preston@@cs.rice.edu\\ 
\vskip4pt
\rm HTML scrap generator by John D. Ramsdell\\ 
\tt ramsdell@@mitre.org\\ 
\vskip4pt
\rm {\tt plain} {\TeX} generation and other enhancements by John Hurst\\ 
\tt ajh@@cs.monash.edu.au
\vskip14pt
\large\rm 14 July 1996
}

\vfootnote{$^\dagger$}{\small\rm This work has been supported by ARPA,
through ONR grant N00014-91-J-1989.\hfil}
\vfill\eject
\endgroup

\input biblio

\input contents
\pageno=-1
\tableofcontents
\vfill\eject

\input numbered
\pageno=1

\def\em{\it}
\def\nutweb{{\tt nutweb}}
\input latexname

\descriptionmargin=30pt

\section{Introduction}

In 1984, Knuth introduced the idea of {\em literate programming\/} and
described a pair of tools to support the practice~\cite{knuth84}.
His approach was to combine Pascal code with \TeX\ documentation to
produce a new language, \verb|WEB|, that offered programmers a superior
approach to programming. He wrote several programs in \verb|WEB|,
including \verb|weave| and \verb|tangle|, the programs used to support
literate programming.
The idea was that a programmer wrote one document, the web file, that
combined documentation (written in \TeX~\cite{texbook}) with code
(written in Pascal).

Running \verb|tangle| on the web file would produce a complete
Pascal program, ready for compilation by an ordinary Pascal compiler.
The primary function of \verb|tangle| is to allow the programmer to
present elements of the program in any desired order, regardless of
the restrictions imposed by the programming language. Thus, the
programmer is free to present his program in a top-down fashion,
bottom-up fashion, or whatever seems best in terms of promoting
understanding and maintenance.

Running \verb|weave| on the web file would produce a \TeX\ file, ready
to be processed by \TeX. The resulting document included a variety of
automatically generated indices and cross-references that made it much
easier to navigate the code. Additionally, all of the code sections
were automatically pretty printed, resulting in a quite impressive
document. 

Knuth also wrote the programs for \TeX\ and {\small\sf METAFONT}
entirely in \verb|WEB|, eventually publishing them in book
form~\cite{texprogram},\cite{metafontprogram}. These are probably the
largest programs ever published in a readable form.

Inspired by Knuth's example, many people have experimented with
\verb|WEB|. Some people have even built web-like tools for their
own favorite combinations of programming language and typesetting
language. For example, \verb|CWEB|, Knuth's current system of choice,
works with a combination of C (or C++) and \TeX~\cite{levy90}.
Another system, FunnelWeb, is independent of any programming language
and only mildly dependent on \TeX~\cite{funnelweb}. Inspired by the
versatility of FunnelWeb and by the daunting size of its
documentation, I (PB) decided to write my own, very simple, tool for
literate programming.%
\footnote{$^1$}{\small\rm There is another system similar to mine,
written by Norman Ramsey, called {\em noweb}~\cite{noweb}. It perhaps
suffers from being overly Unix-dependent and requiring several
programs to use. On the other hand, its command syntax is very nice.
In any case, nuweb certainly owes its name and a number of features to
his inspiration.}

\subsubsection{Nuweb Background}

Nuweb works with any programming language and {\LaTeX} \cite{latex} or
{\tt plain} \TeX~\cite{tex}.  I (PB) wanted to use \LaTeX\ because it
supports a multi-level sectioning scheme and has facilities for
drawing figures.  I wanted to be able to work with arbitrary
programming languages because my friends and I write programs in many
languages (and sometimes combinations of several languages), {\em
e.g.,} C, Fortran, C++, yacc, lex, Scheme, assembly, Postscript, and
so forth.  I (AJH) wanted to use {\TeX} because a) that's what I'm
more used to, and b) it can be tailored to personal preference more
readily, and besides which, c) it runs faster than {\LaTeX}.  

The need to support arbitrary programming languages has many
consequences:
\begin{description}
\item{No Pretty Printing} Both \verb|WEB| and \verb|CWEB| are able to
  pretty print the code sections of their documents because they
  understand the language well enough to parse it. Since we want to use
  {\em any\/} language, we've got to abandon this feature.
\item{No Index Of Identifiers} Because \verb|WEB| knows about Pascal,
  it is able to construct an index of all the identifiers occurring in
  the code sections (filtering out keywords and the standard type
  identifiers). Unfortunately, this isn't as easy in our case. We don't
  know what an identifiers looks like in each language and we certainly
  don't know all the keywords. (On the other hand, see
  Section~\ref{minorcommands})
\end{description}
Of course, we've got to have some compensation for our losses or the
whole idea would be a waste. Here are the advantages I can see:
\begin{description}
\item{Simplicity} The majority of the commands in \verb|WEB| are
  concerned with control of the automatic pretty printing. Since we
  don't pretty print, many commands are eliminated. A further set of
  commands is subsumed by \TeX\  and may also be eliminated. As a
  result, our set of commands is reduced to only four members (explained
  in the next section). This simplicity is also reflected in
  the size of this tool, which is quite a bit smaller than the tools
  used with other approaches.
\item{No Pretty Printing} Everyone disagrees about how their code
  should look, so automatic formatting annoys many people. One approach
  is to provide ways to control the formatting. Our approach is
  simpler---we perform no automatic formatting and therefore allow the
  programmer complete control of code layout.
\item{Control} We also offer the programmer complete control of the
  layout of his output files (the files generated during tangling). Of
  course, this is essential for languages that are sensitive to layout;
  but it is also important in many practical situations, {\em e.g.,}
  debugging.
\item{Speed} Since nuweb doesn't do too much, the nuweb tool runs
  quickly. I combine the functions of \verb|tangle| and \verb|weave| into
  a single program that performs both functions at once.
\item{Page Numbers} Inspired by the example of noweb, nuweb refers to
  all scraps by page number to simplify navigation. If there are
  multiple scraps on a page (say page~17), they are distinguished by
  lower-case letters ({\em e.g.,} 17a, 17b, and so forth).
\item{Multiple File Output} The programmer may specify more than one
  output file in a single nuweb file. This is required when constructing
  programs in a combination of languages (say, Fortran and C). It's also
  an advantage when constructing very large programs that would require
  a lot of compile time.
\end{description}
This last point is very important. By allowing the creation of
multiple output files, we avoid the need for monolithic programs.
Thus we support the creation of very large programs by groups of
people. 

A further reduction in compilation time is achieved by first
writing each output file to a temporary location, then comparing the
temporary file with the old version of the file. If there is no
difference, the temporary file can be deleted. If the files differ,
the old version is deleted and the temporary file renamed. This
approach works well in combination with \verb|make| (or similar tools),
since \verb|make| will avoid recompiling untouched output files.

\subsubsection{Nuweb and HTML}

In addition to producing {\TeX} source, nuweb can be used to
generate HyperText Markup Language (HTML), the markup language used by
the World Wide Web.  HTML provides hypertext links.  When a HTML
document is viewed online, a user can navigate within the document by
activating the links.  The tools which generate HTML automatically
produce hypertext links from a nuweb source.

\subsubsection{Nuweb and \TeX}

I (AJH) choose to modify nuweb after exploring noweb in some detail.
Following Norman Ramsey's article in Software \cite{noweb}, I used
noweb to write documentation for a programming language implementation
(Napier88, \cite{napier}) upon which I had been working, and found its
style to be very pleasing, at least to my eye.  I particularly wanted
a language independent web, because of that different environment, and
the fact that I was dealing in both C code for the run-time
environment, and Napier88 code for much of the compiler
implementation.

However, the pipelined implementation of noweb makes it very slow, a
particular trial for large systems, and I looked about for better
implementations.  Interestingly, I did consider nuweb at this stage,
but rejected it as it was based upon {\LaTeX}, which I did not like.
(The reasons for this are highly subjective, and relate to the fact
that {\LaTeX} is too heavy for my taste, and too difficult to change!)
One important aspect of noweb was that it handled indices of
identifiers, an attribute that I considered essential in documenting
larger systems.

I went through several cycles of noweb, first using it with Robert
Partington's {\TeX} macros, then writing my own, then getting a
student to write the page number generation part into a separate noweb
filter, rather than relying upon cumbersome {\TeX} macros to do so.
Given that noweb's behaviour is much faster when not having to
generate indices, I was convinced that the cross-referencing process
was penalizing the {\TeX} phase significantly.

Intriguingly, we designed this system independently of nuweb, only
later discovering that this is effectively what nuweb does.  Although
it did improve the speed slightly, by about 10\%, the need to
occasionally run the weave phase twice (\'a la nuweb) destroyed any
advantage this gave.  The reason that the speed improvement was so
limited was that although cross-referencing handling was improved, the
need for an additional element in the filter offset this advantage.
Since the speed of this revised noweb was still nothing like {\tt
CWEB} or FunnelWeb, I re-examined the field to see if something
else would suit.

Overlooking the {\LaTeX} part of nuweb, I ran some experiments with
it, and looked at its code in more detail, a delightfully easy thing
to do, when it was so well written.  (That's literate programming for
you!)  For a 1200 line web file, noweb took 16.55 seconds to weave the
document file, while nuweb took only 0.46 seconds.  These figures were
measured on a Silicon Graphics Indy workstation.  The file markup was
changed to reflect the different input notations used by the two
systems.

By changing only a few lines in section \ref{texfile}, and adding an
additional file of macros, I have retargeted the document file to
interface to {\tt plain} {\TeX}.  Of course, the formatting facilities
of {\tt plain} {\TeX} are much more limited than for {\LaTeX}, but for
those who like ``doing their own thing'', this is no real
disadvantage.  The new system that results I have called ``nutweb''
(NUWEB for Tex).\footnote{$^2$}{I shall probably also derive another version
called ``notweb'' (NOWEB for Tex is not noweb) that uses the input
language of noweb, which I admit I prefer to the original {\tt WEB}
style. See also Preston's comment above.}  Needless to say, the system
runs even faster!  Using nutweb and my own suite of macros to do the
formatting, and using nu(t)web itself to test the times, I can now
weave the 3600 line nutweb file in 1.10 seconds (same as nuweb), while
the {\TeX} processing takes only 5.74 seconds compared to {\LaTeX}'s
seconds.

The nutweb system has been rewritten in itself, and this forms a
convenient basis for comparison of the two systems, nuweb and nutweb.

$$\vbox{\halign{\tabskip=1em#\hfil&\hfil#\hfil&\hfil#\hfil&\hskip20pt\hfil#\hfil&\hfil#\hfil\cr
&\multispan2\hfil Sun SPARC 2\hfil&\multispan2\hfil Personal Indy\hfil\cr
nutweb -o nutweb.w&1.26&0.16&0.60&0.08\cr
mytex nutweb.tex&27.40&1.57&13.97&0.61\cr
\cr
nutweb -ol nutweb.lw&1.16&0.21&0.66&0.09\cr
latex nutweb.tex&67.03&1.46&49.55&1.62\cr
\cr
nuweb -o nutweb.lw&0.88&0.21&0.51&0.08\cr
latex nutweb.tex&60.96&1.51&40.48&1.62\cr
}}$$

Due to different installations, the SPARC figures are taken with {\LaTeX}
version 2.0.9, while the Indy figures are taken using \LaTeX 2$\epsilon$.

Nutweb requires some additional macros to work: you should add an
\verb"\input nutmacs.tex" to the beginning of your web file.  I did at
one stage automatically generate this line at the start of the
generated tex file, but this seemed a little gratuitous, and also
meant that you couldn't use nutweb files in other contexts, so it was
removed. The nutmacs macros are defined within the code section of this
document.  Note that these macros also redefine \verb"\bye", so if you
want to mess around with \verb"\bye", be warned!

In the rest of this document, I haven't changed references to Nuweb to
Nutweb, unless Nutweb has some added functionality over Nuweb.

\subsection{Writing Nuweb}

The bulk of a nuweb file will be ordinary {\LaTeX} (or {\TeX}). In
fact, any {\TeX} file can serve as input to nuweb and will be simply
copied through unchanged to the documentation file---unless a nuweb
command is discovered. All nuweb commands begin with an ``at-sign''
(\verb|@@|).  Therefore, a file without at-signs will be copied
unchanged.  Nuweb commands are used to specify {\em output files,}
define {\em macros,} and delimit {\em scraps}. These are the basic
features of interest to the nuweb tool---all else is simply text to be
copied to the documentation file.

\subsubsection{The Major Commands}

Files and macros are defined with the following commands:
\begin{description}
\item{{\tt @@o} {\em file-name flags scrap\/}} Output a file. The file name is
  terminated by whitespace.
\item{{\tt @@d} {\em macro-name scrap\/}} Define a macro. The macro name
  is terminated by a return or the beginning of a scrap.
\end{description}
A specific file may be specified several times, with each definition
being written out, one after the other, in the order they appear.
The definitions of macros may be similarly divided.

\subsubsubsection{Scraps}

Scraps have specific begin markers and end markers to allow precise
control over the contents and layout. Note that any amount of
whitespace (including carriage returns) may appear between a name and
the beginning of a scrap.
\begin{description}
\setbox0\hbox{\verb'@@{'}\setbox1\hbox{\verb'@@}'}
\item{\box0{\em anything\/}\box1} where the scrap body includes every
  character in {\em anything\/}---all the blanks, all the tabs, all the
  carriage returns.
\end{description}
Inside a scrap, we may invoke a macro.
\begin{description}
\item{\verb'@@<'{\em macro-name\/}\verb'@@>'} Causes the macro 
  {\em macro-name\/} to be expanded inline as the code is written out
  to a file. It is an error to specify recursive macro invocations.
\end{description}
Nutweb extends Nuweb by allowing references to macros within
documentation, just as WEB does.

Note that macro names may be abbreviated, either during invocation or
definition. For example, it would be very tedious to have to
repeatedly type the macro name
\begin{narrower}
\verb|@@d Check for terminating at-sequence and return name if found|
\end{narrower}
Therefore, we provide a mechanism (stolen from Knuth) of indicating
abbreviated names.
\begin{narrower}
\verb|@@d Check for terminating...|
\end{narrower}
Basically, the programmer need only type enough characters to uniquely
identify the macro name, followed by three periods. An abbreviation
may even occur before the full version; nuweb simply preserves the
longest version of a macro name. Note also that blanks and tabs are
insignificant in a macro name; any string of them are replaced by a
single blank.

When scraps are written to a program file or a documentation file, tabs are
expanded into spaces by default. Currently, I assume tab stops are set
every eight characters. Furthermore, when a macro is expanded in a scrap,
the body of the macro is indented to match the indentation of the
macro invocation. Therefore, care must be taken with languages 
({\em e.g.,} Fortran) that are sensitive to indentation.
These default behaviors may be changed for each output file (see
below).

\subsubsubsection{Side-by-Side Scraps}

This is a new feature of {\nutweb}.  I (AJH) have often thought
that the basic idea of alternate code and documentation scraps is
good, but in some cases, a little too ``heavyweight''.  For example,
in explaining the purpose of various variable declarations, we often
want a comment alongside each declaration, but adding comments at the
language level seems out of keeping with the literate programming
philosophy.  Knuth goes to some trouble in WEB and CWEB to parse and
typeset comments, but that is inappropriate in a language-less system
like {\nutweb}, and anyway, I didn't like (C)WEB's treatment of
comments.

Whay {\nutweb} introduces is the notion of a {\it side-by-side} scrap.
Instead of {\it vertical} separation of code and documentation, a
side-by-side scrap uses {\it horizontal} separation, and formats the
code in a fixed width verbatim format, and the documentation in normal
typesetting mode.  A vertical rule separates the two fragments, which
are placed side by side (hence the name).  Multiple lines can be
handled, and the longer of the two determines the resultant vertical
space used.

The start of a side-by-side scrap is signalled by a nested \verb"@@{"
within a normal scrap.  The first newline following the nested
\verb"@@{" is skipped (if present), and remaining lines are appended
to the current scrap as if the \verb"@@{" were not present.  

The start of the documentation scrap is indicated by a \verb"@@|"
(mnemonic: the vertical bar represents the vertical rule placed
between the side-by-side code and documentation scraps).  Following
text is formatted in what {\TeX} calls ``restricted vertical mode'',
and is treated (to all intents and purposes) as normal documentation.

The end of a side-by-side section is indicated by a matching
\verb"@@}" that pairs with the nested \verb"@@{" that started it.  A
single following newline (if present) is skipped, and remaining
material is handled as normal code scrap material, and is appended to
that produced by the nested side-by-side code scrap (and the previously
enclosing scrap).  A subsequent \verb"@@}" terminates the entire
scrap.  Note that a user-defined identifier section started by a
\verb"@@|" may still be used.

For example, the following code:
\begin{verbatim}
@@d side-by-side example @@{
  code line 1;
  code line 2;
@@{
  code line 3;
  code line 4;
@@| code lines 3 and 4 are part of a nested ``side-by-side'' code
scrap; they appear contiguous and in order with code lines 1 and 2, 5
and 6.  
@@}
  code line 5;
  code line 6;
@@| code line @@}
\end{verbatim}
will produce
\begingroup\def\codeside{0.4}\def\docoside{0.6}
@d side-by-side example @{
  code line 1;
  code line 2;
@{
  code line 3;
  code line 4;
@| code lines 3 and 4 are part of a nested ``side-by-side'' code
scrap; they appear contiguous and in order with code lines 1 and 2, 5
and 6.
@}
  code line 5;
  code line 6;
@| code line @}
\endgroup
Note that this documentation exceeds the vertical length of
lines 3 and 4, but the result is formatted sensibly.

\subsubsubsection{Flags}

When defining an output file, the programmer has the option of using
flags to control output of a particular file. The flags are intended
to make life a little easier for programmers using certain languages.
They introduce little language dependences; however, they do so only
for a particular file. Thus it is still easy to mix languages within a
single document. There are three ``per-file'' flags:
\begin{description}
\item{\tt -d} Forces the creation of \verb|#line| directives in the
  output file. These are useful with C (and sometimes C++ and Fortran) on
  many Unix systems since they cause the compiler's error messages to
  refer to the web file rather than the output file. Similarly, they
  allow source debugging in terms of the web file.
\item{\tt -i} Suppresses the indentation of macros. That is, when a
  macro is expanded in a scrap, it will {\em not\/} be indented to
  match the indentation of the macro invocation. This flag would seem
  most useful for Fortran programmers.
\item{\tt -t} Suppresses expansion of tabs in the output file. This
  feature seems important when generating \verb|make| files.
\end{description}


\subsubsection{The Minor Commands}\define{minorcommands}

We have two very low-level utility commands that may appear anywhere
in the web file.
\begin{description}
\item{\tt @@@@} Causes a single ``at sign'' to be copied into the output.
\item{\tt @@i {\em file-name\/}} Includes a file. Includes may be
  nested, though there is currently a limit of 10~levels. The file name
  should be complete (no extension will be appended) and should be
  terminated by a carriage return.
\end{description}
Finally, there are three commands used to create indices to the macro
names, file definitions, and user-specified identifiers.
\begin{description}
\item{\tt @@f} Create an index of file names.
\item{\tt @@m} Create an index of macro name.
\item{\tt @@u} Create an index of user-specified identifiers.
\end{description}
I usually put these in their own section
in the \TeX\ document; for example, see Chapter~\ref{indices}.

Identifiers must be explicitly specified for inclusion in the
\verb|@@u| index. By convention, each identifier is marked at the
point of its definition; all references to each identifier (inside
scraps) will be discovered automatically. To ``mark'' an identifier
for inclusion in the index, we must mention it at the end of a scrap.
For example,
\begin{verbatim}
@@d a scrap @@{
Let's pretend we're declaring the variables FOO and BAR
inside this scrap.
@@| FOO BAR @@}
\end{verbatim}
I've used alphabetic identifiers in this example, but any string of
characters (not including whitespace or \verb|@@| characters) will do.
Therefore, it's possible to add index entries for things like
\verb|<<=| if desired. An identifier may be declared in more than one
scrap.

Within the typeset display of a scrap, each identifier so declared has
the defining scrap(s) page reference appended as a subscript.  This
allows ready cross referencing of the definitions of such identifiers.
Contrary to what Preston says in the previous paragraph, this
subscripting only works for (C-style) identifiers.  (Yeah, I know, a
bit of a kludge ... AJH)

In the generated index, each identifier appears with a list of all the
scraps using and defining it, where the defining scraps are
distinguished by underlining. Note that the identifier doesn't
actually have to appear in the defining scrap; it just has to be in
the list of definitions at the end of a scrap.


\subsection{Running Nuweb}

Nuweb is invoked using the following command:
\begin{quote}
{\tt nuweb} {\em flags file-name}$\ldots$
\end{quote}
Nutweb has the same format:
\begin{quote}
{\tt nutweb} {\em flags file-name}$\ldots$
\end{quote}
One or more files may be processed at a time. If a file name has no
extension, \verb|.w| will be appended and the default document
processor ({\LaTeX} for Nuweb, {\TeX} for Nutweb) output produced.
Alternatively, a particular format may be forced by using a suitable
extension, or appropriate command line switch.
\begin{description}
\item{.hw} Will produce output suitable for translation into HTML by
{\LaTeX}2HTML.
\item{.lw} Will produce output suitable for typesetting by {\LaTeX}
(Nutweb only).
\item{.tw} Will produce output suitable for typesetting by {\TeX}
(Nutweb only).
\end{description}
Note that markup otherwise applied to a particular Nutweb file (such as
\verb"\documentstyle" may predicate which processor is used.  Mixed
files may be given for a single invocation of Nutweb.

While a file name may specify a file in another directory, the
resulting documentation file will always be created in the current
directory. For example,
\begin{quote}
{\tt nuweb /foo/bar/quux}
\end{quote}
will take as input the file \verb|/foo/bar/quux.w| and will create the
file \verb|quux.tex| in the current directory.

By default, nuweb performs both tangling and weaving at the same time.
Normally, this is not a bottleneck in the compilation process;
however, it's possible to achieve slightly faster throughput by
avoiding one or another of the default functions using command-line
flags. There are currently four possible flags:
\begin{description}
\item{\tt -t} Suppress generation of the documentation file.
\item{\tt -o} Suppress generation of the output files.
\item{\tt -c} Avoid testing output files for change before updating
them.
\item{\tt -m file} Create only the file named {\tt file} ({\tt nutweb} only). 
\end{description}
Thus, the command
\begin{quote}
\verb|nuweb -to /foo/bar/quux|
\end{quote}
would simply scan the input and produce no output at all.

There are four additional command-line flags:
\begin{description}
\item{\tt -v} For ``verbose,'' causes nuweb to write information about
  its progress to \verb|stderr|.
\item{\tt -l} Force {\LaTeX} style output ({\tt nutweb} only).
\item{\tt -n} Forces scraps to be numbered sequentially from~1
  (instead of using page numbers). This form is perhaps more desirable
  for small webs.
\item{\tt -w} ({\tt nutweb} only) For ``write files'', causes nutweb to
output a message when a file is actually updated.  This is useful for
make scripts that need to know which files have changed.
\item{\tt -x} ({\tt nutweb} only) Turn off the cross reference tagging
for user defined identifiers.
\end{description}

\subsection{Generating HTML}

Nikos Drakos' {\LaTeX}2HTML Version 0.5.3~\cite{drakos94} can be used
to translate {\LaTeX} with embedded HTML scraps into HTML.  Be sure
to include the document-style option \verb|html| so that {\LaTeX} will
understand the hypertext commands.  When translating into HTML, do not
allow a document to be split by specifying ``\verb|-split 0|''.  
You need not generate navigation links, so also specify
``\verb|-no_navigation|''.

While preparing a web, you may want to view the program's scraps without
taking the time to run {\LaTeX}2HTML.  Simply rename the generated
{\LaTeX} source so that its file name ends with \verb|.html|, and view
that file.  The documentations section will be jumbled, but the
scraps will be clear.

\subsection{Restrictions}

Because nuweb is intended to be a simple tool, I've established a few
restrictions. Over time, some of these may be eliminated; others seem
fundamental.
\begin{itemize}
\item The handling of errors is not completely ideal. In some cases, I
  simply warn of a problem and continue; in other cases I halt
  immediately. This behavior should be regularized.
\item I warn about references to macros that haven't been defined, but
  don't halt. This seems most convenient for development, but may change
  in the future.
\item File names and index entries should not contain any \verb|@@|
  signs.
\item Macro names may be (almost) any well-formed \TeX\ string.
  It makes sense to change fonts or use math mode; however, care should
  be taken to ensure matching braces, brackets, and dollar signs.
  When producing HTML, macros are displayed in a preformatted element
  (PRE), so macros may contain one or more A, B, I, U, or P elements
  or data characters.
\item Anything is allowed in the body of a scrap; however, very
  long scraps (horizontally or vertically) may not typeset well.
\item Temporary files (created for comparison to the eventual
  output files) are placed in the current directory. Since they may be
  renamed to an output file name, all the output files should be on the
  same file system as the current directory.
\item Because page numbers cannot be determined until the document has
  been typeset, we have to rerun nuweb after \TeX\ to obtain a clean
  version of the document (very similar to the way we sometimes have
  to rerun \TeX\ to obtain an up-to-date table of contents after
  significant edits).  Nuweb will warn (in most cases) when this needs
  to be done; in the remaining cases, \TeX\ will warn that labels
  may have changed.
\end{itemize}
Very long scraps may be allowed to break across a page if declared
with \verb|@@O| or \verb|@@D| (instead of \verb|@@o| and \verb|@@d|).
This doesn't work very well as a default, since far too many short
scraps will be broken across pages; however, as a user-controlled
option, it seems very useful.  No distinction is made between the
upper case and lower case forms of these commands when generating
HTML.

\subsection{Additional Features added by Nutweb (AJH)}
\begin{itemize}
\item Nutweb doesn't collapse if miscellaneous \verb"@@" sequences
appear.  A message is issued, and it tries to struggle on, generally
successfully.  It is still possible that some sequences can cause
Nutweb to lose its way.

\item Nutweb has a \verb"-w" flag that indicates which output files
were actually (re)written.  This is useful for tracking modified
files, particularly where \verb"make" is involved.

\item Nutweb allows a \verb"@@<...@@>" sequence to appear within the
documentation.  It is replaced by a macro reference, including page
number.

\item Nutweb distinguishes between Nuweb and Nutweb files by the use of
different file suffixes.  This has allowed the integration of the
plain {\TeX} output, similar to the HTML interface.  The default {\tt
.w} extension assumes {\TeX} formatting, but the {\tt -l} flag can be
used to force {\LaTeX} formatting.  Hence {\tt nutweb -l} is
equivalent to {\tt nuweb}, apart from the {\tt nutweb} extensions.

\item Nutweb allows single file extraction (tangling) through the use
of the {\tt -m} switch.  This is useful in building {\tt
make} scripts that must rebuild certain files, without necessarily
redoing other parts that don't need rebuilding.  Although nuweb
doesn't touch files that are unchanged, it will needless regenerate
other files that may not be required by the particular make in
progress.

\item The issue of line breaks at the start and end of scraps 
bothered me.  I have munged the behaviour so that {\it one} initial
line break at the start of a scrap is ignored.  This means that
concatenated scrap fragments don't get line breaks between them,
without having to make the first line contiguous with the scrap
heading.  Incidentally, this is another reason I don't like {\tt
noweb} (which sounds like a double negative, but isn't!).  You cannot
do this sort of thing:
\begin{verbatim}
<<funny one liner>>=<<funny>><<example>>

<<funny>>=All this is one

<<example>>=line
\end{verbatim}
and get the tangled output:
\begin{verbatim}
All this is one line
\end{verbatim}
because of the explicit need for line breaks to flag the end of a
macro body.  I did contemplate switching to noweb syntax, but this
problem spelt the end of the idea.  In nutweb, the above example canbe
done:
\begin{verbatim}
@@d funny example @@{@@<funny@@>@@<example@@>@@}
@@d funny @@{All this is one @@}
@@d example @@{line@@}
\end{verbatim}

With the revised behaviour, the following is equivalent:
\begin{verbatim}
@@d funny example @@{
@@<funny@@>@@<example@@>@@}
@@d funny @@{
All this is one @@}
@@d example @@{
line@@}
\end{verbatim}
Note that breaks at the end of the macro body are still included.

\item Nutweb generates a subscript for identifiers that have been
declared by the user with the \verb"@@|" command.  This allows
convenient cross referencing without having to refer to the identifier
cross-reference listing.  It can be turned off with the {\tt -x} flag.
\end{itemize}

\subsection{Possible Improvements and Enhancements (AJH)}
\begin{itemize}
\item nuweb requires an up-to-date {\tt .aux} file to determine page
numbers.  It outputs a message saying that it must be rerun after
(la)tex.  A way of building this into a make file is warranted.

\item One should rewrite nutweb in itself, just to prove the point,
and provide timing comparisons.

\item What about parameterized macros?  This is a general literate
programming issue.

\item I haven't tested the HTML interface since rewriting the {\TeX}
interface. 

\item do more timing experiments.

\item write all this up in a Technical Report.

\item What about source language support?  Preston has added the
\verb"-d" flag to generate \verb"#line" lines, which is C specific,
and rather {\it ad-hoc}, but what about other languages?  More thought
needed on this problem.

\item input web markup notation could be changed to be more like noweb
(which I prefer).  Again, think carefully on this one.

\item emacs support

\item Nutweb has improved error handling over Nuweb, but there is
still room for improvement.

\item an identifier filter, borrowing upon noweb ideas.  This could be
standalone, and independent of nuweb processing.

\item {\LaTeX} issues a warning: {\tt Label(s) may have
changed. Rerun to get cross-references right.}  nutweb/nutmacs should
do the same.

\end{itemize}

\subsection{Revision History}

\begin{description}\itemskip=0pt
\item{1.3}add cross referencing subscript to user defined identifiers.
\item{1.4}Convert this document to {\tt nutweb}.
\item{1.5}Add the side-by-side formatting, for both {\LaTeX} and {\TeX}.
\end{description}

\subsection{Acknowledgements}

Several people have contributed their times, ideas, and debugging
skills. In particular, I'd like to acknowledge the contributions of
Osman Buyukisik, Manuel Carriba, Adrian Clarke, Tim Harvey, Michael
Lewis, Walter Ravenek, Rob Shillingsburg, Kayvan Sylvan, Dominique
de~Waleffe, and Scott Warren.  Of course, most of these people would
never have heard or nuweb (or many other tools) without the efforts of
George Greenwade.


\ifshowcode
\section{The Overall Structure}

Processing a web requires three major steps:
\begin{enumerate}
\item Read the source, accumulating file names, macro names, scraps,
and lists of cross-references.
\item Reread the source, copying out to the documentation file, with
protection and cross-reference information for all the scraps.
\item Traverse the list of files names. For each file name:
\begin{enumerate}
\item Dump all the defining scraps into a temporary file. 
\item If the file already exists and is unchanged, delete the
temporary file; otherwise, rename the temporary file.
\end{enumerate}
\end{enumerate}


\subsection{Files}

I have divided the program into several files for quicker
recompilation during development.
@o global.h
@{@<Include files@>
@<Type declarations@>
@<C Macro definitions@>
@<Global variable declarations@>
@<Function prototypes@>
@}

We'll need at least three of the standard system include files.
@d Include files
@{#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>
@| FILE stderr exit fprintf fputs fopen fclose getc putc strlen
toupper isupper islower isgraph isspace tempnam remove malloc size_t @}

I also like to use \verb|TRUE| and \verb|FALSE| in my code.
I'd use an \verb|enum| here, except that some systems seem to provide
definitions of \verb|TRUE| and \verb|FALSE| be default.  The following
code seems to work on all the local systems.
@d Type dec...
@{#ifndef FALSE
#define FALSE 0
#endif
#ifndef TRUE
#define TRUE 1
#endif
@| FALSE TRUE @}


\subsubsection{The Main Files}

The code is divided into four main files (introduced here) and five
support files (introduced in the next section).
The file \verb|main.c| will contain the driver for the whole program
(see Section~\ref{mainroutine}).
@o main.c
@{#include "global.h"
@}

The first pass over the source file is contained in \verb|pass1.c|.
It handles collection of all the file names, macros names, and scraps
(see Section~\ref{passone}).
@o pass1.c
@{#include "global.h"
@}

The \verb|.tex| file is created during a second pass over the source
file. The file \verb|tex.c| contains the code controlling the
construction of the \verb|.tex| file 
(see Section~\ref{texfile}).
@o tex.c 
@{#include "global.h"
@}

The \verb|.tex| file is created during a second pass over the source
file. The file \verb|latex.c| contains the code controlling the
construction of the \verb|.tex| file 
(see Section~\ref{latexfile}).
@o latex.c 
@{#include "global.h"
@}

The file \verb|html.c| contains the code controlling the
construction of the \verb|.tex| file appropriate for use with {\LaTeX}2HTML
(see Section~\ref{htmlfile}).
@o html.c
@{#include "global.h"
@}

The code controlling the creation of the output files is in \verb|output.c|
(see Section~\ref{outputfiles}).
@o output.c
@{#include "global.h"
@}

\subsubsection{Support Files}

The support files contain a variety of support routines used to define
and manipulate the major data abstractions.
The file \verb|input.c| holds all the routines used for referring to
source files (see Section~\ref{sourcefiles}).
@o input.c
@{#include "global.h"
@}

Creation and lookup of scraps is handled by routines in \verb|scraps.c|
(see Section~\ref{scraps}).
@o scraps.c
@{#include "global.h"
@}


The handling of file names and macro names is detailed in \verb|names.c|
(see Section~\ref{names}).
@o names.c
@{#include "global.h"
@}

Memory allocation and deallocation is handled by routines in \verb|arena.c|
(see Section~\ref{memorymanagement}).
@o arena.c
@{#include "global.h"
@}

Finally, for best portability, I seem to need a file containing
(useless!) definitions of all the global variables.
@o global.c
@{#include "global.h"
@<Global variable definitions@>
@}

\subsubsection{C Macro Definitions}

Here are some useful macros to control the switch between {\LaTeX} and
{\TeX} code generation.

@d C Macro definitions @{
#define TEX(s) if (tex_flag) fputs(s,tex_file)
#define LATEX(s) if (latex_flag) fputs(s,tex_file)
@}

\subsection{The Main Routine} \define{mainroutine}

The main routine is quite simple in structure.
It wades through the optional command-line arguments,
then handles any files listed on the command line.
@o main.c
@{int main(argc, argv)
     int argc;
     char **argv;
{
  int arg = 1;
  @<Interpret command-line arguments@>
  @<Process the remaining arguments (file names)@>
  exit(0);
}
@| main @}


\subsubsection{Command-Line Arguments}

There are eight possible command-line arguments:
\begin{description}
\item{\tt -c} Forces output files to overwrite old files of the same
  name without comparing for equality first.
\item{\tt -m file} Tangle only the named file.
\item{\tt -n} Forces sequential numbering of scraps (instead of page numbers).
\item{\tt -o} Suppresses generation of the output files.
\item{\tt -t} Suppresses generation of the {\tt .tex} file.
\item{\tt -v} The verbose flag. Forces output of progress reports.
\item{\tt -w} The write flag. Force output message when output file
changes.
\item{\tt -x} The cross referencing flag.  Turn off cross reference
tagging for user defined identifiers.
\end{description}

Global flags are declared for each of the arguments.
\begingroup\def\codeside{0.4}\def\docoside{0.6}
@d Global variable dec... @{
@{extern int doc_flag;      @|if {\tt FALSE}, don't emit the documentation file@}
@{extern int tex_flag;      @| if {\tt TRUE}, emit \TeX\ scraps.@}
@{extern int latex_flag;    @| if {\tt TRUE}, emit \LaTeX\ scraps.@}
@{extern int html_flag;     @| if {\tt TRUE}, emit HTML scraps.@}
@{extern int output_flag;   @| if {\tt FALSE}, don't emit the output files@}
@{extern int compare_flag;  @| if {\tt FALSE}, overwrite without comparison@}
@{extern int verbose_flag;  @| if {\tt TRUE}, write progress information@}
@{extern int write_flag;    @| if {\tt TRUE}, write output file changes@}
@{extern int number_flag;   @| if {\tt TRUE}, use sequential numbering scheme@}
@{extern int make_flag;     @| if {\tt TRUE}, make only a single output file@}
@{extern int cross_flag;    @| if {\tt TRUE}, append defining scrap number to ids@}
@| doc_flag html_flag output_flag compare_flag verbose_flag
number_flag make_flag cross_flag
@}
\endgroup

The flags are all initialized for correct default behavior.  You can
change which document processor is used as default by setting the
appropriate \verb"tex"/\verb"latex"/\verb"html" flag to \verb"TRUE",
and the other two to \verb"FALSE".

@d Global variable def...
@{int doc_flag = TRUE;
int tex_flag = TRUE;
int latex_flag = FALSE;
int html_flag = FALSE;
int output_flag = TRUE;
int compare_flag = TRUE;
int verbose_flag = FALSE;
int write_flag = FALSE;
int number_flag = FALSE;
int make_flag = FALSE;
int cross_flag = TRUE;
@}


We save the invocation name of the command in a global variable
\verb|command_name| for use in error messages.  If a single tangled
file is required, save its name in \verb"make_name"
@d Global variable dec...
@{extern char *command_name;
extern char *make_name;
@| command_name make_name @}

@d Global variable def...
@{char *command_name = NULL;
char *make_name = NULL;
@}

The invocation name is conventionally passed in \verb|argv[0]|.
@d Interpret com...
@{command_name = argv[0];
@}

We need to examine the remaining entries in \verb|argv|, looking for
command-line arguments.
@d Interpret com...
@{while (arg < argc) {
  char *s = argv[arg];
  if (*s++ == '-') {
    @<Interpret the argument string \verb|s|@>
    arg++;
  }
  else break;
}@}


Several flags can be stacked behind a single minus sign; therefore,
we've got to loop through the string, handling them all.  One
exception to this is the single make flag {\tt -m}, which collects a
following file name parameter.
@d Interpret the...
@{{
  char c = *s++;
  while (c) {
    switch (c) {
      case 'c': compare_flag = FALSE; break;
      case 'l': latex_flag = TRUE; tex_flag = FALSE; break;
      case 'm': make_flag = TRUE; make_name = argv[++arg]; break;
      case 'n': number_flag = TRUE; break;
      case 'o': output_flag = FALSE; break;
      case 't': doc_flag = FALSE; break;
      case 'v': verbose_flag = TRUE; break;
      case 'w': write_flag = TRUE; break;
      case 'x': cross_flag = FALSE; break;
      default:  fprintf(stderr, "%s: unexpected argument ignored.  ",
			command_name);
		fprintf(stderr, "Usage is: %s [-cnotvw] file...\n",
			command_name);
		break;
    }
    c = *s++;
  }
}@}

\subsubsection{File Names}

We expect at least one file name. While a missing file name might be
ignored without causing any problems, we take the opportunity to report
the usage convention.
@d Process the remaining...
@{{
  if (arg >= argc) {
    fprintf(stderr, "%s: expected a file name.  ", command_name);
    fprintf(stderr, "Usage is: %s [-cnotv] file-name...\n", command_name);
    exit(-1);
  }
  do {
    @<Handle the file name in \verb|argv[arg]|@>
    arg++;
  } while (arg < argc);
}@}

The code to handle a particular file name is rather more tedious than
the actual processing of the file. A file name may be an arbitrarily
complicated path name, with an optional extension. If no extension is
present, we add \verb|.w| as a default. The extended path name will be
kept in a local variable \verb|source_name|. The resulting documentation
file will be written in the current directory; its name will be kept
in the variable \verb|tex_name|.
@d Handle the file...
@{{
  char source_name[100];
  char tex_name[100];
  char aux_name[100];
  @<Build \verb|source_name| and \verb|tex_name|@>
  @<Process a file@>
}@}


I bump the pointer \verb|p| through the characters in \verb|argv[arg]|,
copying all the characters into \verb|source_name| (via the pointer
\verb|q|). 

At each slash, I update \verb|trim| to point just past the
slash in \verb|source_name|. The effect is that \verb|trim| will point
at the file name without any leading directory specifications.

The pointer \verb|dot| is made to point at the file name extension, if
present. If there is no extension, we add \verb|.w| to the source name.
In any case, we create the \verb|tex_name| from \verb|trim|, taking
care to get the correct extension.  The \verb|html_flag| is set in
this scrap.
@d Build \verb|sou...
@{{
  char *p = argv[arg];
  char *q = source_name;
  char *trim = q;
  char *dot = NULL;
  char c = *p++;
  while (c) {
    *q++ = c;
    if (c == '/') {
      trim = q;
      dot = NULL;
    }
    else if (c == '.')
      dot = q - 1;
    c = *p++;
  }
  *q = '\0';
  if (dot) {
    *dot = '\0'; /* produce HTML when the file extension is ".hw" */
    html_flag = dot[1] == 'h' && dot[2] == 'w' && dot[3] == '\0';
    if (dot[1] == 't' && dot[2] == 'w' && dot[3] == '\0') tex_flag = 1;
    if (dot[1] == 'l' && dot[2] == 'w' && dot[3] == '\0') latex_flag = 1;
    sprintf(tex_name, "%s.tex", trim);
    sprintf(aux_name, "%s.aux", trim);
    *dot = '.';
  }
  else {
    sprintf(tex_name, "%s.tex", trim);
    sprintf(aux_name, "%s.aux", trim);
    *q++ = '.';
    *q++ = 'w';
    *q = '\0';
  }
}@}

Now that we're finally ready to process a file, it's not really too
complex.  We bundle most of the work into four routines \verb|pass1|
(see Section~\ref{passone}), \verb|write_latex| (see
Section~\ref{latexfile}), \verb|write_tex| (see
Section~\ref{texfile}), \verb|write_html| (see
Section~\ref{htmlfile}), and \verb|write_files| (see
Section~\ref{outputfiles}). After we're finished with a
particular file, we must remember to release its storage (see
Section~\ref{memorymanagement}).  The sequential numbering of scraps
is forced when generating HTML.
@d Process a file
@{{
  pass1(source_name);
  if (doc_flag) {
    if (html_flag) {
      int saved_number_flag = number_flag; 
      number_flag = TRUE;
      collect_numbers(aux_name);
      write_html(source_name, tex_name);
      number_flag = saved_number_flag;
    } else {
      collect_numbers(aux_name);
      if (tex_flag) write_tex(source_name, tex_name);
      if (latex_flag) write_latex(source_name, tex_name);
    }
  }
  if (output_flag)
    write_files(file_names);
  arena_free();
}@}


\subsection{Pass One} \define{passone}

During the first pass, we scan the file, recording the definitions of
each macro and file and accumulating all the scraps. 

@d Function pro...
@{extern void pass1();
@}


The routine \verb|pass1| takes a single argument, the name of the
source file. It opens the file, then initializes the scrap structures
(see Section~\ref{scraps}) and the roots of the file-name tree, the
macro-name tree, and the tree of user-specified index entries (see 
Section~\ref{names}). After completing all the
necessary preparation, we make a pass over the file, filling in all
our data structures. Next, we seach all the scraps for references to
the user-specified index entries. Finally, we must reverse all the
cross-reference lists accumulated while scanning the scraps.
@o pass1.c
@{void pass1(file_name)
     char *file_name;
{
  if (verbose_flag)
    fprintf(stderr, "reading %s\n", file_name);
  source_open(file_name);
  init_scraps();
  macro_names = NULL;
  file_names = NULL;
  user_names = NULL;
  @<Scan the source file, looking for at-sequences@>
  if (doc_flag)
    search();
  @<Reverse cross-reference lists@>
}
@| pass1 @}

The only thing we look for in the first pass are the command
sequences. All ordinary text is skipped entirely.
@d Scan the source file, look...
@{{
  int c = source_get();
  while (c != EOF) {
    if (c == '@@')
      @<Scan at-sequence@>
    c = source_get();
  }
}@}


Only four of the at-sequences are interesting during the first pass.
We skip past others immediately; warning if unexpected sequences are
discovered.
@d Scan at-sequence
@{{
  c = source_get();
  switch (c) {
    case 'O':
    case 'o': @<Build output file definition@>
	      break;
    case 'D':
    case 'd': @<Build macro definition@>
	      break;
    case '@@':
    case '<': case '>': /* allow macro refs in documentation */
    case 'u':
    case 'm':
    case 'f': /* ignore during this pass */
	      break;
    default:  fprintf(stderr,
		      "%s: unexpected @@%c sequence ignored (%s, line %d)\n",
		      command_name, c, source_name, source_line);
	      break;
  }
}@}

\subsubsection{Accumulating Definitions}

There are three steps required to handle a definition:
\begin{enumerate}
\item Build an entry for the name so we can look it up later.
\item Collect the scrap and save it in the table of scraps.
\item Attach the scrap to the name.
\end{enumerate}
We go through the same steps for both file names and macro names.
@d Build output file definition
@{{
  Name *name = collect_file_name(); /* returns a pointer to the name entry */
  int scrap = collect_scrap();	    /* returns an index to the scrap */
  @<Add \verb|scrap| to...@>
}@}


@d Build macro definition
@{{
  Name *name = collect_macro_name();
  int scrap = collect_scrap();
  @<Add \verb|scrap| to...@>
}@}


Since a file or macro may be defined by many scraps, we maintain them
in a simple linked list. The list is actually built in reverse order,
with each new definition being added to the head of the list.
@d Add \verb|scrap| to \verb|name|'s definition list
@{{
  Scrap_Node *def = (Scrap_Node *) arena_getmem(sizeof(Scrap_Node));
  def->scrap = scrap;
  def->next = name->defs;
  name->defs = def;
}@}


\subsubsection{Fixing the Cross References}

Since the definition and reference lists for each name are accumulated
in reverse order, we take the time at the end of \verb|pass1| to
reverse them all so they'll be simpler to print out prettily.
The code for \verb|reverse_lists| appears in Section~\ref{names}.
@d Reverse cross-reference lists
@{{
  reverse_lists(file_names);
  reverse_lists(macro_names);
  reverse_lists(user_names);
}@}


\subsection{Writing the Latex File} \define{latexfile}

The second pass (invoked via a call to \verb|write_latex|) copies most of
the text from the source file straight into a \verb|.tex| file.
Definitions are formatted slightly and cross-reference information is
printed out.

Note that all the formatting is handled in this section.
If you don't like the format of definitions or indices or whatever,
it'll be in this section somewhere. Similarly, if someone wanted to
modify nuweb to work with a different typesetting system, this would
be the place to look.  (I looked here, and it {\it was} very straightforward
to change: AJH)

@d Function...
@{extern void write_latex();
@}

We need a few local function declarations before we get into the body
of \verb|write_latex|.

@o latex.c @{
@{static void copy_scrap();		@| formats the body of a scrap @}
@{static void print_scrap_numbers();	@| formats a list of scrap numbers @}
@{static void format_entry();		@| formats an index entry @}
static void format_user_entry();
@}


The routine \verb|write_latex| takes two file names as parameters: the
name of the web source file and the name of the \verb|.tex| output file.
@o latex.c
@{void write_latex(file_name, tex_name)
     char *file_name;
     char *tex_name;
{
  FILE *tex_file = fopen(tex_name, "w");
  if (tex_file) {
    if (verbose_flag)
      fprintf(stderr, "writing %s\n", tex_name);
    source_open(file_name);
    @<Copy \verb|source_file| into \verb|tex_file|@>
    fclose(tex_file);
  }
  else
    fprintf(stderr, "%s: can't open %s\n", command_name, tex_name);
}
@| write_latex @}


We make our second (and final) pass through the source web, this time
copying characters straight into the \verb|.tex| file. However, we keep
an eye peeled for \verb|@@|~characters, which signal a command sequence.

@d Copy \verb|source_file| into \verb|tex_file|
@{{
  int scraps = 1;
  int c = source_get();
  while (c != EOF) {
    if (c == '@@')
      @<Interpret at-sequence@>
    else {
      putc(c, tex_file);
      c = source_get();
    }
  }
}@}

@d Interpret at-sequence
@{{
  int big_definition = FALSE;
  c = source_get();
  switch (c) {
    case 'O': big_definition = TRUE;
    case 'o': @<Write output file definition@>
	      break;
    case 'D': big_definition = TRUE;
    case 'd': @<Write macro definition@>
	      break;
    case 'f': @<Write index of file names@>
	      break;
    case 'm': @<Write index of macro names@>
	      break;
    case 'u': @<Write user-name index@>
	      break;
    case '@@': putc(c, tex_file);
    default:  c = source_get();
	      break;
  }
}@}


\subsubsection{Formatting Definitions}

We go through a fair amount of effort to format a file definition.
I've derived most of the \LaTeX\ commands experimentally; it's quite
likely that an expert could do a better job. The \LaTeX\ for
the previous macro definition should look like this (perhaps modulo
the scrap references):
{\small
\begin{verbatim}
\begin{flushleft} \small
\begin{minipage}{\linewidth} \label{scrap40}
$\langle$Interpret at-sequence {\footnotesize 20}$\rangle\equiv$
\vspace{-1ex}
\begin{list}{}{\advance\textwidth by -\leftmargin}\item
\mbox{}\verb@@@@\\
\mbox{}\verb@@  int big_definition = FALSE@@$_{10a}$\verb@@;@@\\
\mbox{}\verb@@  c = source_get@@$_{57c}$\verb@@();@@\\
\mbox{}\verb@@  switch (c) {@@\\
\mbox{}\verb@@    case 'O': big_definition = TRUE@@$_{10a}$\verb@@;@@\\
\vdots
\mbox{}\verb@@    case 'u': @@$\langle$Write user-name index {\footnotesize 29a}$\rangle$\verb@@@@\\
\mbox{}\verb@@              break;@@\\
\mbox{}\verb@@    case '@@{\tt @@}\verb@@': putc@@$_{9b}$\verb@@(c, tex_file);@@\\
\mbox{}\verb@@    default:  c = source_get@@$_{57c}$\verb@@();@@\\
\mbox{}\verb@@              break;@@\\
\mbox{}\verb@@  }@@\\
\mbox{}\verb@@}@@$\diamond$
\end{list}
\vspace{-1ex}
\footnotesize\addtolength{\baselineskip}{-1ex}
\begin{list}{}{\setlength{\itemsep}{-\parsep}\setlength{\itemindent}{-\leftmargin}}
\item Macro referenced in scrap 19b.
\end{list}
\end{minipage}\\[4ex]
\end{flushleft}
\end{verbatim}}

\noindent
The {\em flushleft\/} environment is used to avoid \LaTeX\ warnings
about underful lines. The {\em vbox\/} environment is used to
avoid page breaks in the middle of scraps. The {\em verb\/} command
allows arbitrary characters to be printed (however, note the special
handling of the \verb|@@| case in the switch statement).

Macro and file definitions are formatted nearly identically.
I've factored the common parts out into separate scraps.

@d Write output file definition
@{{
  Name *name = collect_file_name();
  @<Begin the scrap environment@>
  fprintf(tex_file, "\\verb@@\"%s\"@@ {\\footnotesize ", name->spelling);
  write_single_scrap_ref(tex_file, scraps++);
  fputs(" }$\\equiv$\n", tex_file);
  @<Fill in the middle of the scrap environment@>
  @<Write file defs@>
  @<Finish the scrap environment@>
}@}


I don't format a macro name at all specially, figuring the programmer
might want to use italics or bold face in the midst of the name.

@d Write macro definition
@{{
  Name *name = collect_macro_name();
  @<Begin the scrap environment@>
  fprintf(tex_file, "$\\langle$%s {\\footnotesize ", name->spelling);
  write_single_scrap_ref(tex_file, scraps++);
  fputs("}$\\rangle\\equiv$\n", tex_file);
  @<Fill in the middle of the scrap environment@>
  @<Write macro defs@>
  @<Write macro refs@>
  @<Finish the scrap environment@>
}@}


@d Begin the scrap environment
@{{
  fputs("\\begin{flushleft} \\small", tex_file);
  if (!big_definition)
    fputs("\n\\begin{minipage}{\\linewidth}", tex_file);
  fprintf(tex_file, " \\label{scrap%d}\n", scraps);
}@}

The interesting things here are the $\diamond$ inserted at the end of
each scrap and the various spacing commands. The diamond helps to
clearly indicate the end of a scrap. The spacing commands were derived
empirically; they may be adjusted to taste.

Note also the whitespace eater. I use it to remove any blank lines
that appear after a scrap in the source file. This way, text following
a scrap will not be indented. Again, this is a matter of personal taste.

@d Finish the scrap environment
@{{
  if (!big_definition)
    fputs("\\end{minipage}\\\\[4ex]\n", tex_file);
  fputs("\\end{flushleft}\n", tex_file);
  do
    c = source_get();
  while (isspace(c));
}@}


\subsubsubsection{Formatting Cross References}

@d Write macro defs
@{{
  fputs("\\vspace{-1ex}\n", tex_file);
  fputs("\\footnotesize\\addtolength{\\baselineskip}{-1ex}\n", tex_file);
  fputs("\\begin{list}{}{\\setlength{\\itemsep}{-\\parsep}", tex_file);
  fputs("\\setlength{\\itemindent}{-\\leftmargin}}\n", tex_file);
  if (name->defs->next) {
    fputs("\\item Macro defined by scraps ", tex_file);
    print_scrap_numbers(tex_file, name->defs);
  }
}@}

@d Write macro refs
@{{
  if (name->uses) {
    if (name->uses->next) {
      fputs("\\item Macro referenced in scraps ", tex_file);
      print_scrap_numbers(tex_file, name->uses);
    }
    else {
      fputs("\\item Macro referenced in scrap ", tex_file);
      write_single_scrap_ref(tex_file, name->uses->scrap);
      fputs(".\n", tex_file);
    }
  }
  else {
    fputs("\\item Macro never referenced.\n", tex_file);
    fprintf(stderr, "%s: <%s> never referenced.\n",
	    command_name, name->spelling);
  }
  fputs("\\end{list}\n", tex_file);
}@}


@o latex.c
@{static void print_scrap_numbers(tex_file, scraps)
     FILE *tex_file;
     Scrap_Node *scraps;
{
  int page;
  write_scrap_ref(tex_file, scraps->scrap, TRUE, &page);
  scraps = scraps->next;
  while (scraps) {
    write_scrap_ref(tex_file, scraps->scrap, FALSE, &page);
    scraps = scraps->next;
  }
  fputs(".\n", tex_file);
}
@| print_scrap_numbers @}


\subsubsubsection{Formatting a Scrap}

We add a \verb|\mbox{}| at the beginning of each line to avoid
problems with older versions of \TeX.

@o latex.c
@{static void copy_scrap(c,file)
    char c;
    FILE *file;
{
  int indent = 0; 
  int codestate = 0;
  int verbmode=0;
  while (1) {
    switch (c) {
      case '@@':  @<Check at-sequence for end-of-scrap@>
		 break;
      case '\n': 
                 if (codestate!=2 && !verbmode) {
                   verbmode=1; fputs("\\mbox{}\\verb@@", file);
                 }
                 if (codestate!=2) {
                   verbmode=0; fputs("@@\\\\\n", file); indent = 0;
                 } else if (codestate==2) fputs("\n", file);
                 break;
      case '\t': 
                 if (codestate!=2 && !verbmode) {
                   verbmode=1; fputs("\\mbox{}\\verb@@", file);
                 }
                 @<Expand tab into spaces@>
		 break;
      default:
                 if (codestate!=2 && !verbmode) {
                   verbmode=1; fputs("\\mbox{}\\verb@@", file);
                 }
                 if (cross_flag && isalpha(c)) {@<check for identifier@>}
                 else {putc(c, file); indent++; break;}
    }
    c = source_get();
  }
}
@| copy_scrap @}


@d Expand tab into spaces
@{{
  int delta = 8 - (indent % 8);
  indent += delta;
  while (delta > 0) {
    putc(' ', file);
    delta--;
  }
}@}

@d Check at-sequence...
@{{
  c = source_get();
  switch (c) {
    case '@@': if (verbmode) fputs("@@{\\tt @@}\\verb@@", file);
              else fputs("{\\tt @@}", file);
	      break;
    case '|': if (codestate==1) {
                codestate=2;
                @<Switch a side-by-side format@>
              } else {
                if (verbmode) putc('@@',file);
                @<Skip over index entries@>
                return;
              }
              break;
    case '}': if (codestate==0) {
                if (verbmode) putc('@@', file); return;
              } else if (codestate==1) {
                if (verbmode) putc('@@', file); codestate=0; 
                break;
              } else if (codestate==2) {
                @<Close a side-by-side format@>
              }
              break;
    case '{': if (codestate==0) {@<Open a side-by-side format@>}
              break;
    case '<': @<Format macro name@>
	      break;
    default:  /* ignore these since pass1 will have warned about them */
	      break;
  }
}@}

In the {\LaTeX} version, I have hard coded the relative widths of code
and documentation in the side by side format.  This is because there
is no easy way to enter a new parameter to {\LaTeX} without creating a
new style, or inputting some commands.  I (AJH) wanted to keep the {\LaTeX}
part as simple as possible, consistent with Preston's philosophy.

Perhaps some {\LaTeX} {\TeX}pert might suggest a better way?

@d Open a side-by-side format @{
{
  codestate=1;
  if (verbmode) putc('@@',file);
  fputs("\\begingroup\\catcode`\\^^M=5%\n",file);
  fputs("\\setbox0\\vtop{\\hsize=0.6\\textwidth\\raggedright%\n",file);
@{
  if (source_peek=='\n') c = source_get();
@|
  note that after opening a
  side-by-side code section, we consume a single trailing newline: 
  this is consistent with a normal code opening.
@}
}
@}

Here is more hard coded stuff; again, AJH records his apologies.

@d Switch a side-by-side format @{{
if (verbmode) putc('@@',file);
fputs("}\\setbox1\\vtop{\\hsize=0.4\\textwidth\\leftskip=10pt\n",file);
verbmode=0;
}@}

@d Close a side-by-side format @{{
codestate=0; verbmode=0;
fputs("}\\ifnum\\dp1>\\dp0\\dp0=\\dp1\\else\\dp1=\\dp0\\fi\n",file);
fputs("\\hbox{\\box0\\vrule\\box1}\n",file);
fputs("\\endgroup",file);
if (source_peek=='\n') c = source_get();
}@}

There's no need to check for errors here, since we will have already
pointed out any during the first pass.
@d Skip over index entries
@{{
  do {
    do
      c = source_get();
    while (c != '@@');
    c = source_get();
  } while (c != '}');
}@}


@d Format macro name
@{{
  Name *name = collect_scrap_name();
  if (verbmode) putc('@@',file);
  fprintf(file, "$\\langle$%s {\\footnotesize ", name->spelling);
  if (name->defs)
    @<Write abbreviated definition list@>
  else {
    putc('?', file);
    fprintf(stderr, "%s: scrap never defined <%s>\n",
	    command_name, name->spelling);
  }
  fputs("}$\\rangle$", file);
  if (verbmode) fputs("\\verb@@",file);
}@}


@d Write abbreviated definition list
@{{
  Scrap_Node *p = name->defs;
  write_single_scrap_ref(file, p->scrap);
  p = p->next;
  if (p)
    fputs(", \\ldots\\ ", file);
}@}


\subsubsection{Generating the Indices}

@d Write index of file names
@{{
  if (file_names) {
    fputs("\n{\\small\\begin{list}{}{\\setlength{\\itemsep}{-\\parsep}",
    	  tex_file);
    fputs("\\setlength{\\itemindent}{-\\leftmargin}}\n", tex_file);
    format_entry(file_names, tex_file, TRUE);
    fputs("\\end{list}}", tex_file);
  }
  c = source_get();
}@}


@d Write index of macro names
@{{
  if (macro_names) {
    fputs("\n{\\small\\begin{list}{}{\\setlength{\\itemsep}{-\\parsep}",
    	  tex_file);
    fputs("\\setlength{\\itemindent}{-\\leftmargin}}\n", tex_file);
    format_entry(macro_names, tex_file, FALSE);
    fputs("\\end{list}}", tex_file);
  }
  c = source_get();
}@}


@o latex.c
@{static void format_entry(name, tex_file, file_flag)
     Name *name;
     FILE *tex_file;
     int file_flag;
{
  while (name) {
    format_entry(name->llink, tex_file, file_flag);
    @<Format an index entry@>
    name = name->rlink;
  }
}
@| format_entry @}


@d Format an index entry
@{{
  fputs("\\item ", tex_file);
  if (file_flag) {
    fprintf(tex_file, "\\verb@@\"%s\"@@ ", name->spelling);
    @<Write file's defining scrap numbers@>
  }
  else {
    fprintf(tex_file, "$\\langle$%s {\\footnotesize ", name->spelling);
    @<Write defining scrap numbers@>
    fputs("}$\\rangle$ ", tex_file);
    @<Write referencing scrap numbers@>
  }
  putc('\n', tex_file);
}@}


@d Write file's defining scrap numbers
@{{
  Scrap_Node *p = name->defs;
  fputs("{\\footnotesize Defined by scrap", tex_file);
  if (p->next) {
    fputs("s ", tex_file);
    print_scrap_numbers(tex_file, p);
  }
  else {
    putc(' ', tex_file);
    write_single_scrap_ref(tex_file, p->scrap);
    putc('.', tex_file);
  }
  putc('}', tex_file);
}@}

@d Write defining scrap numbers
@{{
  Scrap_Node *p = name->defs;
  if (p) {
    int page;
    write_scrap_ref(tex_file, p->scrap, TRUE, &page);
    p = p->next;
    while (p) {
      write_scrap_ref(tex_file, p->scrap, FALSE, &page);
      p = p->next;
    }
  }
  else
    putc('?', tex_file);
}@}

@d Write referencing scrap numbers
@{{
  Scrap_Node *p = name->uses;
  fputs("{\\footnotesize ", tex_file);
  if (p) {
    fputs("Referenced in scrap", tex_file);
    if (p->next) {
      fputs("s ", tex_file);
      print_scrap_numbers(tex_file, p);
    }
    else {
      putc(' ', tex_file);
      write_single_scrap_ref(tex_file, p->scrap);
      putc('.', tex_file);
    }
  }
  else
    fputs("Not referenced.", tex_file);
  putc('}', tex_file);
}@}


@d Write user-name index
@{{
  if (user_names) {
    fputs("\n{\\small\\begin{list}{}{\\setlength{\\itemsep}{-\\parsep}",
    	  tex_file);
    fputs("\\setlength{\\itemindent}{-\\leftmargin}}\n", tex_file);
    format_user_entry(user_names, tex_file);
    fputs("\\end{list}}", tex_file);
  }
  c = source_get();
}@}


@o latex.c
@{static void format_user_entry(name, tex_file)
     Name *name;
     FILE *tex_file;
{
  while (name) {
    format_user_entry(name->llink, tex_file);
    @<Format a user index entry@>
    name = name->rlink;
  }
}
@| format_user_entry @}


@d Format a user index entry
@{{
  Scrap_Node *uses = name->uses;
  if (uses) {
    int page;
    Scrap_Node *defs = name->defs;
    fprintf(tex_file, "\\item \\verb@@%s@@: ", name->spelling);
    if (uses->scrap < defs->scrap) {
      write_scrap_ref(tex_file, uses->scrap, TRUE, &page);
      uses = uses->next;
    }
    else {
      if (defs->scrap == uses->scrap)
        uses = uses->next;
      fputs("\\underline{", tex_file);
      write_single_scrap_ref(tex_file, defs->scrap);
      putc('}', tex_file);
      page = -2;
      defs = defs->next;
    }
    while (uses || defs) {
      if (uses && (!defs || uses->scrap < defs->scrap)) {
        write_scrap_ref(tex_file, uses->scrap, FALSE, &page);
        uses = uses->next;
      }
      else {
        if (uses && defs->scrap == uses->scrap)
	  uses = uses->next;
        fputs(", \\underline{", tex_file);
        write_single_scrap_ref(tex_file, defs->scrap);
        putc('}', tex_file);
        page = -2;
        defs = defs->next;
      }
    }
    fputs(".\n", tex_file);
  }
}@}


\subsection{Writing the Tex File}\define{texfile}

This section is a more or less verbatim copy of the previous section,
with the macro names changed to have a prepended \verb"tex" label.  As
commonalities are identified, I will remove them.

The second pass (invoked via a call to \verb|write_tex|) copies most of
the text from the source file straight into a \verb|.tex| file.
Definitions are formatted slightly and cross-reference information is
printed out.

Note that all the formatting is handled in this section.  If you don't
like the format of definitions or indices or whatever, it'll be in
this section somewhere. Similarly, if someone wanted to modify nuweb
to work with a different typesetting system, this would be the place
to look.  (I looked here, and it was very straightforward to change:
AJH)

@d Function...
@{extern void write_tex();
extern Scrap_Node *get_defs();
@}

We need a few local function declarations before we get into the body
of \verb|write_tex|.

@o tex.c @{
@{static void copy_scrap();		@| formats the body of a scrap @}
@{static void print_scrap_numbers();	@| formats a list of scrap numbers @}
@{static void format_entry();		@| formats an index entry @}
static void format_user_entry();
@}


The routine \verb|write_tex| takes two file names as parameters: the
name of the web source file and the name of the \verb|.tex| output file.
@o tex.c
@{void write_tex(file_name, tex_name)
     char *file_name;
     char *tex_name;
{
  FILE *tex_file = fopen(tex_name, "w");
  if (tex_file) {
    if (verbose_flag)
      fprintf(stderr, "writing %s\n", tex_name);
    source_open(file_name);
    @<tex Copy \verb|source_file| into \verb|tex_file|@>
    fclose(tex_file);
  }
  else
    fprintf(stderr, "%s: can't open %s\n", command_name, tex_name);
}
@| write_tex @}


We make our second (and final) pass through the source web, this time
copying characters straight into the \verb|.tex| file. However, we keep
an eye peeled for \verb|@@|~characters, which signal a command sequence.

@d tex Copy \verb|source_file| into \verb|tex_file|
@{{
  int scraps = 1;
  int c = source_get();
  while (c != EOF) {
    if (c == '@@')
      @<tex Interpret at-sequence@>
    else {
      putc(c, tex_file);
      c = source_get();
    }
  }
}@}

Note the funny positioning of the \verb"c = source_get();" statement
in the handling of an \verb"@@<" command.  This is due to the nested
use of a \verb"c" variable, and the fact that this does not properly
advance the input, at least at the outer level.  This should probably
be fixed at some stage, by making the \verb"c" in \verb"c =
source_get();" a global variable.

@d tex Interpret at-sequence
@{{
  int big_definition = FALSE;
  c = source_get();
  switch (c) {
    case 'O': big_definition = TRUE;
    case 'o': @<tex Write output file definition@>
	      break;
    case 'D': big_definition = TRUE;
    case 'd': @<tex Write macro definition@>
	      break;
    case 'f': @<tex Write index of file names@>
	      break;
    case 'm': @<tex Write index of macro names@>
	      break;
    case 'u': @<tex Write user-name index@>
	      break;
    case '<': @<tex Format tex macro name@>
              c = source_get(); /* swallow the < character */
              break;
    case '@@': putc(c, tex_file);
    default:  c = source_get();
	      break;
  }
}@}

\subsubsection{Formatting Definitions}

We go through a fair amount of effort to format a file definition.
I've derived most of the \TeX\ commands experimentally; it's quite
likely that an expert could do a better job. The \TeX\ for
the previous macro definition should look like this (perhaps modulo
the scrap references):
{\small
\begin{verbatim}
\vskip\parskip\vbox{\begingroup\label{scrap69}\parskip=0pt\obeylines%
\advance\leftskip by \codemargin%
\hskip-\codemargin$\langle$tex Interpret at-sequence {\footnotesize 33}$\rangle\equiv$%
\verb@@{@@
\verb@@  int big_definition = FALSE@@$_{10a}$\verb@@;@@
\verb@@  c = source_get@@$_{58b}$\verb@@();@@
\verb@@  switch (c) {@@
\verb@@    case 'O': big_definition = TRUE@@$_{10a}$\verb@@;@@
\verb@@    case 'o': @@$\langle$tex Write output file definition {\footnotesize 34a}$\rangle$\verb@@@@
\verb@@              break;@@
\verb@@    case 'D': big_definition = TRUE@@$_{10a}$\verb@@;@@
\verb@@    case 'd': @@$\langle$tex Write macro definition {\footnotesize 34b}$\rangle$\verb@@@@
\verb@@              break;@@
\verb@@    case 'f': @@$\langle$tex Write index of file names {\footnotesize 41a}$\rangle$\verb@@@@
\verb@@              break;@@
\verb@@    case 'm': @@$\langle$tex Write index of macro names {\footnotesize 41b}$\rangle$\verb@@@@
\verb@@              break;@@
\verb@@    case 'u': @@$\langle$tex Write user-name index {\footnotesize 43a}$\rangle$\verb@@@@
\verb@@              break;@@
\verb@@    case '<': @@$\langle$tex Format tex macro name {\footnotesize 40b}$\rangle$\verb@@@@
\verb@@              c = source_get@@$_{58b}$\verb@@(); /* swallow the < character */@@
\verb@@              break;@@
\verb@@    case '@@{\tt @@}\verb@@': putc@@$_{9b}$\verb@@(c, tex_file);@@
\verb@@    default:  c = source_get@@$_{58b}$\verb@@();@@
\verb@@              break;@@
\verb@@  }@@
\verb@@}@@$\diamond$\par\endgroup%
\begingroup\parskip=0pt\footnotesize\advance\leftskip by \refsmargin%
Macro referenced in scrap 32c.\par
\endgroup}
\end{verbatim}}

\noindent
The {\tt vbox} environment is used to
avoid page breaks in the middle of scraps. The {\tt verb} command
allows arbitrary characters to be printed (however, note the special
handling of the \verb|@@| case in the switch statement).

Macro and file definitions are formatted nearly identically.
I've factored the common parts out into separate scraps.

@d tex Write output file definition
@{{
  Name *name = collect_file_name();
  @<tex Begin the scrap environment@>
  fprintf(tex_file, "\\verb@@\"%s\"@@ {\\footnotesize ", name->spelling);
  write_single_scrap_ref(tex_file, scraps++);
  fputs(" }$\\equiv$%\n", tex_file);
  @<Fill in the middle of the scrap environment@>
  @<Write file defs@>
  @<tex Finish the scrap environment@>
}@}


I don't format a macro name at all specially, figuring the programmer
might want to use italics or bold face in the midst of the name.

@d tex Write macro definition
@{{
  Name *name = collect_macro_name();
  @<tex Begin the scrap environment@>
  fprintf(tex_file, "$\\langle$%s {\\footnotesize ", name->spelling);
  write_single_scrap_ref(tex_file, scraps++);
  fputs("}$\\rangle\\equiv$%\n", tex_file);
  @<Fill in the middle of the scrap environment@>
  @<tex Write macro defs@>
  @<tex Write macro refs@>
  @<tex Finish the scrap environment@>
}@}


@d tex Begin the scrap environment
@{{
  fputs("\\vskip\\parskip", tex_file);
  if (!big_definition)
    fputs("\\vbox{", tex_file);
  fprintf(tex_file, "\\begingroup\\label{scrap%d}\\parskip=0pt", scraps);
  fputs("\\obeylines%\n", tex_file);
  fputs("\\advance\\leftskip by \\codemargin%\n", tex_file);
  fputs("\\hskip-\\codemargin", tex_file);
}@}

The interesting things here are the $\diamond$ inserted at the end of
each scrap and the various spacing commands. The diamond helps to
clearly indicate the end of a scrap. The spacing commands were derived
empirically; they may be adjusted to taste.

We've got one last spacing command, controlling the amount of white
space after a scrap.

Note also the whitespace eater. I use it to remove any blank lines
that appear after a scrap in the source file. This way, text following
a scrap will not be indented. Again, this is a matter of personal taste.

@d tex Finish the scrap environment
@{{
  if (!big_definition) fputs("}", tex_file);
  fputs("\n\n", tex_file);
  do
    c = source_get();
  while (isspace(c));
}@}


\subsubsubsection{Formatting Cross References}

@d tex Write macro defs
@{{
  fputs("\\begingroup\\parskip=0pt\\footnotesize", tex_file);
  fputs("\\advance\\leftskip by \\refsmargin%\n", tex_file);
  if (name->defs->next) {
    fputs("Macro defined by scraps ", tex_file);
    print_scrap_numbers(tex_file, name->defs);
    fputs("\\par\n", tex_file);
  }
}@}

@d tex Write macro refs
@{{
  if (name->uses) {
    if (name->uses->next) {
      fputs("Macro referenced in scraps ", tex_file);
      print_scrap_numbers(tex_file, name->uses);
      fputs("\\par\n", tex_file);
    }
    else {
      fputs("Macro referenced in scrap ", tex_file);
      write_single_scrap_ref(tex_file, name->uses->scrap);
      fputs(".\\par\n", tex_file);
    }
  }
  else {
    fputs("Macro never referenced.\n", tex_file);
    fprintf(stderr, "%s: <%s> never referenced.\n",
	    command_name, name->spelling);
      fputs("\\par\n", tex_file);
  }
  fputs("\\endgroup", tex_file);
}@}


@o tex.c
@{static void print_scrap_numbers(tex_file, scraps)
     FILE *tex_file;
     Scrap_Node *scraps;
{
  int page;
  write_scrap_ref(tex_file, scraps->scrap, TRUE, &page);
  scraps = scraps->next;
  while (scraps) {
    write_scrap_ref(tex_file, scraps->scrap, FALSE, &page);
    scraps = scraps->next;
  }
  fputs(".", tex_file);
}
@| print_scrap_numbers @}


\subsubsubsection{Formatting a Scrap}

Bit of a kludge here: after the start of a scrap, indicated by the
\verb"@@"\verb"{" sequence, we ignore exactly 1 following newline.

While copying a scrap, we are in one of two basic states: verbatim or
normal.  Code is usually typeset in verbatim, except that newlines and
at sequences must escape from verbatim mode.  Hence we use a flag
variable to indicate which state, and when switching, output the
appropriate control sequence: \verb"\verb@@" to move into verbatim,
\verb"@@" to escape.
@o tex.c @{
static void copy_scrap(c,file)
    char c;
    FILE *file;
{
  int indent = 0; int codestate = 0; int verbmode = 0;
  while (1) {
    switch (c) {
      case '@@': @<tex Check at-sequence for end-of-scrap@>
      case '\n': @<tex Handle newline in {\tt copy\_scrap}@>
      case '\t': @<tex Handle tabs in {\tt copy\_scrap}@>
      default: @<tex Default copy in {\tt copy\_scrap}@>
    }
    c = source_get();
  }
}
@| copy_scrap @}

Within a code scrap, at-sequences can start a macro call
(\verb"@@<.."), flag a list of identifiers (\verb"@@|.."), or end the
scrap (\verb"@@}").  In version 1.5, we also add the side-by-side scrap
mechanism: a nested \verb"@@{..@@|..@@}" signals side by side code and
documentation scraps.

This is implemented by the state variable {\tt codestate}, which has
the values:
\begin{description}\def\descrformat#1{\tt#1\hfil}\itemskip=0pt
\item{0}normal code fragment, no special treatment.
\item{1}nested side-by-side code fragment.
\item{2}nested documentation fragment
\end{description}

@d tex Check at-sequence... @{
c = source_get();
switch (c) {
  case '@@': if (verbmode) fputs("@@{\\tt @@}\\verb@@", file);
            else fputs("{\\tt @@}", file);
     break;
  case '|': if (codestate==1) {
              codestate=2;
              @<tex Switch a side-by-side format@>
            } else {
              if (verbmode) putc('@@',file);
              @<tex Skip over index entries@>
              return;
            }
            break;
  case '}': if (codestate==0) {
              if (verbmode) putc('@@', file); return;
            } else if (codestate==1) {
              if (verbmode) putc('@@', file); codestate=0; 
              break;
            } else if (codestate==2) {
              @<tex Close a side-by-side format@>
            }
            break;
  case '{': if (codestate==0) {@<tex Open a side-by-side format@>}
            break;
  case '<': @<tex Format macro name@>
     break;
  default:  /* ignore these since pass1 will have warned about them */
     break;
}
break;
@}

@d tex Open a side-by-side format @{{
codestate=1;
if (verbmode) putc('@@',file);
fputs("\\begingroup\\catcode`\\^^M=5%\n",file);
fputs("\\setbox0\\vtop{\\obeylines\\hsize=\\codeside\\hsize%\n",file);
if (source_peek=='\n') c = source_get();
}@}

@d tex Switch a side-by-side format @{{
if (verbmode) putc('@@',file);
fputs("}\\setbox1\\vtop{\\hsize=\\docoside\\hsize\\leftskip=10pt\n",file);
verbmode=0;
}@}

@d tex Close a side-by-side format @{{
codestate=0;
fputs("}\\ifnum\\dp1>\\dp0\\dp0=\\dp1\\else\\dp1=\\dp0\\fi\n",file);
fputs("\\hbox{\\box0\\vrule\\box1}\n",file);
fputs("\\endgroup",file);
if (source_peek=='\n') c = source_get();
}@}

When we hit a newline, we look to see what state we are in.  If we are
in a code scrap (outer or nested), and not in verbatim mode, we enter
verbatim by issuing a \verb.\verb"@@". If we are in verbatim mode in a
code scrap (we may have just switched to it), we must add the
terminating `\verb"@@"' for the verbatim mode, and then switch out of
verbatim.  Otherwise, in nested documentation, just copy the newline.

@d tex Handle newline in {\tt copy\_scrap} @{
@<tex Switch to verb mode if in code scrap@>
if (codestate!=2) {
  verbmode=0; fputs("@@\n", file); indent = 0;
} else if (codestate==2) fputs("\n", file);
break;
@}

@d tex Handle tabs in {\tt copy\_scrap} @{
@<tex Switch to verb mode if in code scrap@>
@<tex Expand tab into spaces@>
break;
@}

@d tex Expand tab into spaces
@{{
  int delta = 8 - (indent % 8);
  indent += delta;
  while (delta > 0) {
    putc(' ', file);
    delta--;
  }
}@}

@d tex Default copy in {\tt copy\_scrap} @{
@<tex Switch to verb mode if in code scrap@>
if (codestate!=2 && cross_flag && isalpha(c)) {
  @<check for identifier@>}
else {putc(c, file); indent++; break;}
break;
@}

We've found a character that isn't a command.  It should be copied to
the output, but first we must make sure that if we are in a code
scrap, that verbatim mode is switched on.

@d tex Switch to verb mode if in code scrap@{
if (codestate!=2 && !verbmode) {
  verbmode=1; fputs("\\verb@@", file);
}
@}

@d check for identifier 
@{{
Scrap_Node *defs;
char spelling[100];
char *sp = spelling;
int first;
int page; page=-2;
while (1) {
  *sp++ = c; putc(c,file); indent++;
  if (!(isalpha(source_peek) || isdigit(source_peek) || source_peek=='_'))
    break;
  c = source_get();
}
*sp = '\0';
defs = get_defs(spelling, user_names);
if (defs) {
  if (verbmode) putc('@@',file);
  fputs("$_{",file);
  @<format definition label@>
  fputs("}$",file);
  if (verbmode) fputs("\\verb@@",file);
}
}
@}

@d format definition label
@{first=1;
while (defs) {
  write_scrap_ref(file,defs->scrap,first,&page);
  first = 0;
  defs = defs->next;
}
@}

There's no need to check for errors here, since we will have already
pointed out any during the first pass.
@d tex Skip over index entries
@{{
  do {
    do
      c = source_get();
    while (c != '@@');
    c = source_get();
  } while (c != '}');
}@}


@d tex Format macro name
@{{
  Name *name = collect_scrap_name();
  if (verbmode) putc('@@',file);
  fprintf(file, "$\\langle$%s {\\footnotesize ", name->spelling);
  if (name->defs)
    @<tex Write abbreviated definition list@>
  else {
    putc('?', file);
    fprintf(stderr, "%s: scrap never defined <%s>\n",
	    command_name, name->spelling);
  }
  fputs("}$\\rangle$", file);
  if (verbmode) fputs("\\verb@@",file);
}@}

This macro does much the same as the last macro, except that it writes to
the \verb"tex_file", and doesn't assume that it appears in the midst
of a code fragment (hence no verbatim escapes).

@d tex Format tex macro name
@{{
  Name *name = collect_scrap_name();
  fprintf(tex_file, "$\\langle$%s {\\footnotesize ", name->spelling);
  if (name->defs)
    @<tex Write tex abbreviated definition list@>
  else {
    putc('?', tex_file);
    fprintf(stderr, "%s: scrap never defined <%s>\n",
	    command_name, name->spelling);
  }
  fputs("}$\\rangle$", tex_file);
}@}


@d tex Write abbreviated definition list
@{{
  Scrap_Node *p = name->defs;
  write_single_scrap_ref(file, p->scrap);
  p = p->next;
  if (p)
    fputs(", $\\ldots$\\ ", file);
}@}

This macro does the same as the last macro, except that it writes to
the \verb"tex_file".

@d tex Write tex abbreviated definition list
@{{
  Scrap_Node *p = name->defs;
  write_single_scrap_ref(tex_file, p->scrap);
  p = p->next;
  if (p)
    fputs(", $\\ldots$\\ ", tex_file);
}@}


\subsubsection{Generating the Indices}

@d tex Write index of file names
@{{
  if (file_names) {
    fputs("\n\\begingroup\\parskip=0pt\n",tex_file);
    format_entry(file_names, tex_file, TRUE);
    fputs("\\endgroup", tex_file);
  }
  c = source_get();
}@}


@d tex Write index of macro names
@{{
  if (macro_names) {
    fputs("\n\\begingroup\\parskip=0pt\n",tex_file);
    format_entry(macro_names, tex_file, FALSE);
    fputs("\\endgroup", tex_file);
  }
  c = source_get();
}@}


@o tex.c
@{static void format_entry(name, tex_file, file_flag)
     Name *name;
     FILE *tex_file;
     int file_flag;
{
  while (name) {
    format_entry(name->llink, tex_file, file_flag);
    @<tex Format an index entry@>
    name = name->rlink;
  }
}
@| format_entry @}


@d tex Format an index entry
@{{
  fputs("\\par ", tex_file);
  if (file_flag) {
    fprintf(tex_file, "\\verb@@\"%s\"@@ ", name->spelling);
    @<tex Write file's defining scrap numbers@>
  }
  else {
    fprintf(tex_file, "$\\langle$%s {\\footnotesize ", name->spelling);
    @<tex Write defining scrap numbers@>
    fputs("}$\\rangle$ ", tex_file);
    @<tex Write referencing scrap numbers@>
  }
  putc('\n', tex_file);
}@}


@d tex Write file's defining scrap numbers
@{{
  Scrap_Node *p = name->defs;
  fputs("{\\footnotesize Defined by scrap", tex_file);
  if (p->next) {
    fputs("s ", tex_file);
    print_scrap_numbers(tex_file, p);
  }
  else {
    putc(' ', tex_file);
    write_single_scrap_ref(tex_file, p->scrap);
    putc('.', tex_file);
  }
  putc('}', tex_file);
}@}

@d tex Write defining scrap numbers
@{{
  Scrap_Node *p = name->defs;
  if (p) {
    int page;
    write_scrap_ref(tex_file, p->scrap, TRUE, &page);
    p = p->next;
    while (p) {
      write_scrap_ref(tex_file, p->scrap, FALSE, &page);
      p = p->next;
    }
  }
  else
    putc('?', tex_file);
}@}

@d tex Write referencing scrap numbers
@{{
  Scrap_Node *p = name->uses;
  fputs("{\\footnotesize ", tex_file);
  if (p) {
    fputs("Referenced in scrap", tex_file);
    if (p->next) {
      fputs("s ", tex_file);
      print_scrap_numbers(tex_file, p);
    }
    else {
      putc(' ', tex_file);
      write_single_scrap_ref(tex_file, p->scrap);
      putc('.', tex_file);
    }
  }
  else
    fputs("Not referenced.", tex_file);
  putc('}', tex_file);
}@}


@d tex Write user-name index
@{{
  if (user_names) {
    fputs("\n\\begingroup\\parskip=0pt\n", tex_file);
    format_user_entry(user_names, tex_file);
    fputs("\\endgroup", tex_file);
  }
  c = source_get();
}@}


@o tex.c
@{static void format_user_entry(name, tex_file)
     Name *name;
     FILE *tex_file;
{
  while (name) {
    format_user_entry(name->llink, tex_file);
    @<tex Format a user index entry@>
    name = name->rlink;
  }
}
@| format_user_entry @}


@d tex Format a user index entry
@{{
  Scrap_Node *uses = name->uses;
  if (uses) {
    int page;
    Scrap_Node *defs = name->defs;
    fprintf(tex_file, "\\par \\verb@@%s@@: ", name->spelling);
    if (uses->scrap < defs->scrap) {
      write_scrap_ref(tex_file, uses->scrap, TRUE, &page);
      uses = uses->next;
    }
    else {
      if (defs->scrap == uses->scrap)
        uses = uses->next;
      fputs("$\\underline{", tex_file);
      write_single_scrap_ref(tex_file, defs->scrap);
      fputs("}$", tex_file);
      page = -2;
      defs = defs->next;
    }
    while (uses || defs) {
      if (uses && (!defs || uses->scrap < defs->scrap)) {
        write_scrap_ref(tex_file, uses->scrap, FALSE, &page);
        uses = uses->next;
      }
      else {
        if (uses && defs->scrap == uses->scrap)
	  uses = uses->next;
        fputs(", $\\underline{", tex_file);
        write_single_scrap_ref(tex_file, defs->scrap);
        fputs("}$", tex_file);
        page = -2;
        defs = defs->next;
      }
    }
    fputs(".\n", tex_file);
  }
}@}

\subsection{Writing the Tex File with HTML Scraps} \define{htmlfile} 

The HTML generated is patterned closely upon the {\TeX} generated in
the previous section.\footnote{$^3$}{While writing this section, I
tried to follow Preston's style as displayed in
Section~\ref{texfile}---J. D. R.}  When a file name ends in
\verb|.hw|, the second pass (invoked via a call to \verb|write_html|)
copies most of the text from the source file straight into a
\verb|.tex| file.  Definitions are formatted slightly and
cross-reference information is printed out.

@d Function...
@{extern void write_html();
@}

We need a few local function declarations before we get into the body
of \verb|write_html|.

@o html.c @{
@{static void copy_scrap();               @| formats the body of a scrap @}
@{static void display_scrap_ref();        @| formats a scrap reference @}
@{static void display_scrap_numbers();    @| formats a list of scrap numbers @}
@{static void print_scrap_numbers();      @| pluralizes scrap formats list @}
@{static void format_entry();             @| formats an index entry @}
static void format_user_entry();
@}


The routine \verb|write_html| takes two file names as parameters: the
name of the web source file and the name of the \verb|.tex| output file.
@o html.c
@{void write_html(file_name, html_name)
     char *file_name;
     char *html_name;
{
  FILE *html_file = fopen(html_name, "w");
  if (html_file) {
    if (verbose_flag)
      fprintf(stderr, "writing %s\n", html_name);
    source_open(file_name);
    @<Copy \verb|source_file| into \verb|html_file|@>
    fclose(html_file);
  }
  else
    fprintf(stderr, "%s: can't open %s\n", command_name, html_name);
}
@| write_html @}


We make our second (and final) pass through the source web, this time
copying characters straight into the \verb|.tex| file. However, we keep
an eye peeled for \verb|@@|~characters, which signal a command sequence.

@d Copy \verb|source_file| into \verb|html_file|
@{{
  int scraps = 1;
  int c = source_get();
  while (c != EOF) {
    if (c == '@@')
      @<Interpret HTML at-sequence@>
    else {
      putc(c, html_file);
      c = source_get();
    }
  }
}@}

@d Interpret HTML at-sequence
@{{
  c = source_get();
  switch (c) {
    case 'O': 
    case 'o': @<Write HTML output file definition@>
	      break;
    case 'D': 
    case 'd': @<Write HTML macro definition@>
	      break;
    case 'f': @<Write HTML index of file names@>
	      break;
    case 'm': @<Write HTML index of macro names@>
	      break;
    case 'u': @<Write HTML user-name index@>
	      break;
    case '@@': putc(c, html_file);
    default:  c = source_get();
	      break;
  }
}@}


\subsubsection{Formatting Definitions}

We go through only a little amount of effort to format a definition.
The HTML for the previous macro definition should look like this
(perhaps modulo the scrap references):

\begin{verbatim}
<pre>
<a name="nuweb68">&lt;Interpret HTML at-sequence 68&gt;</a> =
{
  c = source_get();
  switch (c) {
    case 'O': 
    case 'o': &lt;Write HTML output file definition <a href="#nuweb69">69</a>&gt;
              break;
    case 'D': 
    case 'd': &lt;Write HTML macro definition <a href="#nuweb71">71</a>&gt;
              break;
    case 'f': &lt;Write HTML index of file names <a href="#nuweb86">86</a>&gt;
              break;
    case 'm': &lt;Write HTML index of macro names <a href="#nuweb87">87</a>&gt;
              break;
    case 'u': &lt;Write HTML user-name index <a href="#nuweb93">93</a>&gt;
              break;
    case '@@': putc(c, html_file);
    default:  c = source_get();
              break;
  }
}&lt;&gt;</pre>
Macro referenced in scrap <a href="#nuweb67">67</a>.
<br>
\end{verbatim}

Macro and file definitions are formatted nearly identically.
I've factored the common parts out into separate scraps.

@d Write HTML output file definition
@{{
  Name *name = collect_file_name();
  @<Begin HTML scrap environment@>
  @<Write HTML output file declaration@>
  scraps++;
  @<Fill in the middle of HTML scrap environment@>
  @<Write HTML file defs@>
  @<Finish HTML scrap environment@>
}@}

@d Write HTML output file declaration
@{  fputs("<a name=\"nuweb", html_file);
  write_single_scrap_ref(html_file, scraps);
  fprintf(html_file, "\"><code>\"%s\"</code> ", name->spelling);
  write_single_scrap_ref(html_file, scraps);
  fputs("</a> =\n", html_file);
@}

@d Write HTML macro definition
@{{
  Name *name = collect_macro_name();
  @<Begin HTML scrap environment@>
  @<Write HTML macro declaration@>
  scraps++;
  @<Fill in the middle of HTML scrap environment@>
  @<Write HTML macro defs@>
  @<Write HTML macro refs@>
  @<Finish HTML scrap environment@>
}@}

I don't format a macro name at all specially, figuring the programmer
might want to use italics or bold face in the midst of the name.  Note
that in this implementation, programmers may only use directives in
macro names that are recognized in preformatted text elements (PRE).

@d Write HTML macro declaration
@{  fputs("<a name=\"nuweb", html_file);
  write_single_scrap_ref(html_file, scraps);
  fprintf(html_file, "\">&lt;%s ", name->spelling);
  write_single_scrap_ref(html_file, scraps);
  fputs("&gt;</a> =\n", html_file);
@}

@d Begin HTML scrap environment
@{{
  fputs("\\begin{rawhtml}\n", html_file);
  fputs("<pre>\n", html_file);
}@}

The end of a scrap is marked with the characters \verb|<>|.
@d Fill in the middle of HTML scrap environment
@{{
  copy_scrap(html_file);
  fputs("&lt;&gt;</pre>\n", html_file);
}@}

The only task remaining is to get rid of the current at command and
end the paragraph.

@d Finish HTML scrap environment
@{{
  fputs("\\end{rawhtml}\n", html_file);
  c = source_get(); /* Get rid of current at command. */
}@}


\subsubsubsection{Formatting Cross References}

@d Write HTML file defs
@{{
  if (name->defs->next) {
    fputs("File defined by ", html_file);
    print_scrap_numbers(html_file, name->defs);
    fputs("<br>\n", html_file);
  }
}@}

@d Write HTML macro defs
@{{
  if (name->defs->next) {
    fputs("Macro defined by ", html_file);
    print_scrap_numbers(html_file, name->defs);
    fputs("<br>\n", html_file);
  }
}@}

@d Write HTML macro refs
@{{
  if (name->uses) {
    fputs("Macro referenced in ", html_file);
    print_scrap_numbers(html_file, name->uses);
  }
  else {
    fputs("Macro never referenced.\n", html_file);
    fprintf(stderr, "%s: <%s> never referenced.\n",
	    command_name, name->spelling);
  }
  fputs("<br>\n", html_file);
}@}

@o html.c
@{static void display_scrap_ref(html_file, num)
     FILE *html_file;
     int num;
{
  fputs("<a href=\"#nuweb", html_file);
  write_single_scrap_ref(html_file, num);
  fputs("\">", html_file);
  write_single_scrap_ref(html_file, num);
  fputs("</a>", html_file);
}
@| display_scrap_ref @}

@o html.c
@{static void display_scrap_numbers(html_file, scraps)
     FILE *html_file;
     Scrap_Node *scraps;
{
  display_scrap_ref(html_file, scraps->scrap);
  scraps = scraps->next;
  while (scraps) {
    fputs(", ", html_file);
    display_scrap_ref(html_file, scraps->scrap);
    scraps = scraps->next;
  }
}
@| display_scrap_numbers @}

@o html.c
@{static void print_scrap_numbers(html_file, scraps)
     FILE *html_file;
     Scrap_Node *scraps;
{
  fputs("scrap", html_file);
  if (scraps->next) fputc('s', html_file);
  fputc(' ', html_file);
  display_scrap_numbers(html_file, scraps);
  fputs(".\n", html_file);
}
@| print_scrap_numbers @}


\subsubsubsection{Formatting a Scrap}

We must translate HTML special keywords into entities in scraps.

@o html.c
@{static void copy_scrap(file)
     FILE *file;
{
  int indent = 0;
  int c = source_get();
  while (1) {
    switch (c) {
      case '@@':  @<Check HTML at-sequence for end-of-scrap@>
		 break;
      case '<' : fputs("&lt;", file);
		 indent++;
		 break;
      case '>' : fputs("&gt;", file);
		 indent++;
		 break;
      case '&' : fputs("&amp;", file);
		 indent++;
		 break;
      case '\n': fputc(c, file);
		 indent = 0;
		 break;
      case '\t': @<Expand tab into spaces@>
		 break;
      default:   putc(c, file);
		 indent++;
		 break;
    }
    c = source_get();
  }
}
@| copy_scrap @}

@d Check HTML at-sequence...
@{{
  c = source_get();
  switch (c) {
    case '@@': fputc(c, file);
	      break;
    case '|': @<Skip over index entries@>
    case '}': return;
    case '<': @<Format HTML macro name@>
	      break;
    default:  /* ignore these since pass1 will have warned about them */
	      break;
  }
}@}

There's no need to check for errors here, since we will have already
pointed out any during the first pass.

@d Format HTML macro name
@{{
  Name *name = collect_scrap_name();
  fprintf(file, "&lt;%s ", name->spelling);
  if (name->defs)
    @<Write HTML abbreviated definition list@>
  else {
    putc('?', file);
    fprintf(stderr, "%s: scrap never defined <%s>\n",
	    command_name, name->spelling);
  }
  fputs("&gt;", file);
}@}


@d Write HTML abbreviated definition list
@{{
  Scrap_Node *p = name->defs;
  display_scrap_ref(file, p->scrap);
  if (p->next)
    fputs(", ... ", file);
}@}


\subsubsection{Generating the Indices}

@d Write HTML index of file names
@{{
  if (file_names) {
    fputs("\\begin{rawhtml}\n", html_file);
    fputs("<dl compact>\n", html_file);
    format_entry(file_names, html_file, TRUE);
    fputs("</dl>\n", html_file);
    fputs("\\end{rawhtml}\n", html_file);
  }
  c = source_get();
}@}


@d Write HTML index of macro names
@{{
  if (macro_names) {
    fputs("\\begin{rawhtml}\n", html_file);
    fputs("<dl compact>\n", html_file);
    format_entry(macro_names, html_file, FALSE);
    fputs("</dl>\n", html_file);
    fputs("\\end{rawhtml}\n", html_file);
  }
  c = source_get();
}@}


@o html.c
@{static void format_entry(name, html_file, file_flag)
     Name *name;
     FILE *html_file;
     int file_flag;
{
  while (name) {
    format_entry(name->llink, html_file, file_flag);
    @<Format an HTML index entry@>
    name = name->rlink;
  }
}
@| format_entry @}


@d Format an HTML index entry
@{{
  fputs("<dt> ", html_file);
  if (file_flag) {
    fprintf(html_file, "<code>\"%s\"</code>\n<dd> ", name->spelling);
    @<Write HTML file's defining scrap numbers@>
  }
  else {
    fprintf(html_file, "&lt;%s ", name->spelling);
    @<Write HTML defining scrap numbers@>
    fputs("&gt;\n<dd> ", html_file);
    @<Write HTML referencing scrap numbers@>
  }
  putc('\n', html_file);
}@}


@d Write HTML file's defining scrap numbers
@{{
  fputs("Defined by ", html_file);
  print_scrap_numbers(html_file, name->defs);
}@}

@d Write HTML defining scrap numbers
@{{
  if (name->defs)
    display_scrap_numbers(html_file, name->defs);
  else
    putc('?', html_file);
}@}

@d Write HTML referencing scrap numbers
@{{
  Scrap_Node *p = name->uses;
  if (p) {
    fputs("Referenced in ", html_file);
    print_scrap_numbers(html_file, p);
  }
  else
    fputs("Not referenced.\n", html_file);
}@}


@d Write HTML user-name index
@{{
  if (user_names) {
    fputs("\\begin{rawhtml}\n", html_file);
    fputs("<dl compact>\n", html_file);
    format_user_entry(user_names, html_file);
    fputs("</dl>\n", html_file);
    fputs("\\end{rawhtml}\n", html_file);
  }
  c = source_get();
}@}


@o html.c
@{static void format_user_entry(name, html_file)
     Name *name;
     FILE *html_file;
{
  while (name) {
    format_user_entry(name->llink, html_file);
    @<Format a user HTML index entry@>
    name = name->rlink;
  }
}
@| format_user_entry @}


@d Format a user HTML index entry
@{{
  Scrap_Node *uses = name->uses;
  if (uses) {
    Scrap_Node *defs = name->defs;
    fprintf(html_file, "<dt><code>%s</code>:\n<dd> ", name->spelling);
    if (uses->scrap < defs->scrap) {
      display_scrap_ref(html_file, uses->scrap);
      uses = uses->next;
    }
    else {
      if (defs->scrap == uses->scrap)
	uses = uses->next;
      fputs("<strong>", html_file);
      display_scrap_ref(html_file, defs->scrap);
      fputs("</strong>", html_file);
      defs = defs->next;
    }
    while (uses || defs) {
      fputs(", ", html_file);
      if (uses && (!defs || uses->scrap < defs->scrap)) {
	display_scrap_ref(html_file, uses->scrap);
	uses = uses->next;
      }
      else {
	if (uses && defs->scrap == uses->scrap)
	  uses = uses->next;
	fputs("<strong>", html_file);
	display_scrap_ref(html_file, defs->scrap);
	fputs("</strong>", html_file);
	defs = defs->next;
      }
    }
    fputs(".\n", html_file);
  }
}@}

\subsection{Common Code for Documentation Generation}

In this section we have gathered together some fragments that share
common features in documentation generation.  Specific output
sequences are flagged by the C macros {\tt TEX} and {\tt LATEX}.

I have attempted to follow the same subsection headings.

\subsubsection{Formatting Definitions}

If the scrap starts with a new line, swallow it, issue a \verb"\par"
({\TeX} only), then copy the scrap.

@d Fill in the middle of the scrap environment @{
{
  int c = source_get();
  LATEX("\\vspace{-1ex}\n");
  LATEX("\\begin{list}{}{\\advance\\textwidth by -\\leftmargin}");
  LATEX("\\item\n");
  if (c == '\n') {TEX("\\par%\n"); c = source_get();}
  copy_scrap(c,tex_file);
  LATEX("$\\diamond$\n\\end{list}\n");
  TEX("$\\diamond$\\par\\endgroup%\n");
}
@}

The {\tt copy\_scrap} procedure looks for identifiers, and appends a
definition flag in the form of a subscript scrap number.  This is
found by searching the identifier tree.

\subsubsubsection{Formatting Cross References}

@d Write file defs @{
{
  if (name->defs->next) {
    LATEX("\\vspace{-1ex}\n");
    LATEX("\\footnotesize\\addtolength{\\baselineskip}{-1ex}\n");
    LATEX("\\begin{list}{}{\\setlength{\\itemsep}{-\\parsep}");
    LATEX("\\setlength{\\itemindent}{-\\leftmargin}}\n");
    LATEX("\\item File defined by scraps ");
    TEX("\\begingroup\\parskip=0pt\\footnotesize");
    TEX("\\advance\\leftskip by \\refsmargin%\n");
    TEX("\\par File defined by scraps ");
    print_scrap_numbers(tex_file, name->defs);
    LATEX("\\end{list}\n");
    TEX("\\par\\endgroup");
  } else {
    LATEX("\\vspace{-2ex}\n");
  }
}
@}

\subsubsubsection{Formatting a Scrap}
\subsubsection{Generating the Indices}

\subsection{Writing the Output Files} \define{outputfiles}

@d Function pro...
@{extern void write_files();
@}

Output a file only if the single file flag is not set, or we have a
name match.
@o output.c
@{void write_files(files)
     Name *files;
{
  int do_file;
  while (files) {
    write_files(files->llink);
    do_file = ! make_flag;
    if ( make_flag ) do_file = ! strcmp(make_name,files->spelling);
    if ( do_file ) @<Write out \verb|files->spelling|@>
    files = files->rlink;
  }
}
@| write_files @}

We call \verb|tempnam|, causing it to create a file name in the
current directory.  This could cause a problem for \verb|rename| if
the eventual output file will reside on a different file system.
Perhaps it would be better to examine \verb|files->spelling| to find
any directory information.

Note the superfluous call to \verb|remove| before \verb|rename|.
We're using it get around a bug in some implementations of
\verb|rename|.

@d Write out \verb|files->spelling|
@{{
  char indent_chars[500];
  int file_updated = 0;
  FILE *temp_file;
  char *temp_name = tempnam(".", 0);
  temp_file = fopen(temp_name, "w");
  if (!temp_file) {
    fprintf(stderr, "%s: can't create %s for a temporary file\n",
	    command_name, temp_name);
    exit(-1);
  }  
  if (verbose_flag)
    fprintf(stderr, "writing %s\n", files->spelling);
  write_scraps(temp_file, files->defs, 0, indent_chars,
	       files->debug_flag, files->tab_flag, files->indent_flag);
  fclose(temp_file);
  if (compare_flag)
    @<Compare the temp file and the old file@>
  else {
    file_updated = 1;
    remove(files->spelling);
    rename(temp_name, files->spelling);
  }
  if (write_flag && file_updated)
    fprintf(stderr, "written %s\n", files->spelling);
}@}

Again, we use a call to \verb|remove| before \verb|rename|.
@d Compare the temp file...
@{{
  FILE *old_file = fopen(files->spelling, "r");
  if (old_file) {
    int x, y;
    temp_file = fopen(temp_name, "r");
    do {
      x = getc(old_file);
      y = getc(temp_file);
    } while (x == y && x != EOF);
    fclose(old_file);
    fclose(temp_file);
    if (x == y)
      remove(temp_name);
    else {
      file_updated = 1;
      remove(files->spelling);
      rename(temp_name, files->spelling);
    }
  }
  else
    file_updated = 1;
    rename(temp_name, files->spelling);
}@}



\section{The Support Routines}

\subsection{Source Files} \define{sourcefiles}

\subsubsection{Global Declarations}

We need two routines to handle reading the source files.
@d Function pro...
@{extern void source_open(); /* pass in the name of the source file */
extern int source_get();   /* no args; returns the next char or EOF */
@}


There are also three global variables maintained for use in error
messages and such.
@d Global variable dec...
@{extern char *source_name;  /* name of the current file */
extern int source_line;    /* current line in the source file */
extern int source_peek;    /* next character in source file */
@| source_name source_line @}

@d Global variable def...
@{char *source_name = NULL;
int source_line = 0;
int source_peek = 0;
@}

\subsubsection{Local Declarations}

@o input.c
@{static FILE *source_file;  /* the current input file */
static int double_at;
static int include_depth;
@| source_peek source_file double_at include_depth @}


@o input.c
@{static struct {
  FILE *file;
  char *name;
  int line;
} stack[10];
@| stack @}


\subsubsection{Reading a File}

The routine \verb|source_get| returns the next character from the
current source file. It notices newlines and keeps the line counter 
\verb|source_line| up to date. It also catches \verb|EOF| and watches
for \verb|@@|~characters. All other characters are immediately returned.
@o input.c
@{int source_get()
{
  int c = source_peek;
  switch (c) {
    case EOF:  @<Handle \verb|EOF|@>
	       return c;
    case '@@':  @<Handle an ``at'' character@>
	       return c;
    case '\n': source_line++;
    default:   source_peek = getc(source_file);
	       return c;
  }
}
@| source_get @}


This whole \verb|@@|~character handling mess is pretty annoying.
I want to recognize \verb|@@i| so I can handle include files correctly.
At the same time, it makes sense to recognize illegal \verb|@@|~sequences
and complain; this avoids ever having to check anywhere else.
Unfortunately, I need to avoid tripping over the \verb|@@@@|~sequence;
hence this whole unsatisfactory \verb|double_at| business.
@d Handle an ``at''...
@{{
  c = getc(source_file);
  if (double_at) {
    source_peek = c;
    double_at = FALSE;
    c = '@@';
  }
  else
    switch (c) {
      case 'i': @<Open an include file@>
		break;
      case 'f': case 'm': case 'u':
      case 'd': case 'o': case 'D': case 'O':
      case '{': case '}': case '<': case '>': case '|':
		source_peek = c;
		c = '@@';
		break;
      case '@@': source_peek = c;
		double_at = TRUE;
		break;
      default:  fprintf(stderr, "%s: bad @@%c sequence (%s, line %d)\n",
			command_name, c, source_name, source_line);
                source_peek = c; c = getc(source_file);
		break;/*exit(-1);*/
    }
}@}

@d Open an include file
@{{
  char name[100];
  if (include_depth >= 10) {
    fprintf(stderr, "%s: include nesting too deep (%s, %d)\n",
	    command_name, source_name, source_line);
    exit(-1);
  }
  @<Collect include-file name@>
  stack[include_depth].name = source_name;
  stack[include_depth].file = source_file;
  stack[include_depth].line = source_line + 1;
  include_depth++;
  source_line = 1;
  source_name = save_string(name);
  source_file = fopen(source_name, "r");
  if (!source_file) {
    fprintf(stderr, "%s: can't open include file %s\n",
     command_name, source_name);
    exit(-1);
  }
  source_peek = getc(source_file);
  c = source_get();
}@}

@d Collect include-file name
@{{
    char *p = name;
    do 
      c = getc(source_file);
    while (c == ' ' || c == '\t');
    while (isgraph(c)) {
      *p++ = c;
      c = getc(source_file);
    }
    *p = '\0';
    if (c != '\n') {
      fprintf(stderr, "%s: unexpected characters after file name (%s, %d)\n",
	      command_name, source_name, source_line);
      exit(-1);
    }
}@}

If an \verb|EOF| is discovered, the current file must be closed and
input from the next stacked file must be resumed. If no more files are
on the stack, the \verb|EOF| is returned.
@d Handle \verb|EOF|
@{{
  fclose(source_file);
  if (include_depth) {
    include_depth--;
    source_file = stack[include_depth].file;
    source_line = stack[include_depth].line;
    source_name = stack[include_depth].name;
    source_peek = getc(source_file);
    c = source_get();
  }
}@}


\subsubsection{Opening a File}

The routine \verb|source_open| takes a file name and tries to open the
file. If unsuccessful, it complains and halts. Otherwise, it sets 
\verb|source_name|, \verb|source_line|, and \verb|double_at|.
@o input.c
@{void source_open(name)
     char *name;
{
  source_file = fopen(name, "r");
  if (!source_file) {
    fprintf(stderr, "%s: couldn't open %s\n", command_name, name);
    exit(-1);
  }
  source_name = name;
  source_line = 1;
  source_peek = getc(source_file);
  double_at = FALSE;
  include_depth = 0;
}
@| source_open @}




\subsection{Scraps} \define{scraps}


@o scraps.c
@{#define SLAB_SIZE 500

typedef struct slab {
  struct slab *next;
  char chars[SLAB_SIZE];
} Slab;
@| Slab SLAB_SIZE @}


@o scraps.c
@{typedef struct {
  char *file_name;
  int file_line;
  int page;
  char letter;
  Slab *slab;
} ScrapEntry;
@| ScrapEntry @}

@o scraps.c
@{static ScrapEntry *SCRAP[256];

#define scrap_array(i) SCRAP[(i) >> 8][(i) & 255]

static int scraps;
@| scraps scrap_array SCRAP @}


@d Function pro...
@{extern void init_scraps();
extern int collect_scrap();
extern int write_scraps();
extern void write_scrap_ref();
extern void write_single_scrap_ref();
@}


@o scraps.c
@{void init_scraps()
{
  scraps = 1;
  SCRAP[0] = (ScrapEntry *) arena_getmem(256 * sizeof(ScrapEntry));
}
@| init_scraps @}


@o scraps.c
@{void write_scrap_ref(file, num, first, page)
     FILE *file;
     int num;
     int first;
     int *page;
{
  if (scrap_array(num).page >= 0) {
    if (first)
      fprintf(file, "%d", scrap_array(num).page);
    else if (scrap_array(num).page != *page)
      fprintf(file, ", %d", scrap_array(num).page);
    if (scrap_array(num).letter > 0)
      fputc(scrap_array(num).letter, file);
  }
  else {
    if (first)
      putc('?', file);
    else
      fputs(", ?", file);
    @<Warn (only once) about needing to rerun after Tex@>
  }
  *page = scrap_array(num).page;
}
@| write_scrap_ref @}

@o scraps.c
@{void write_single_scrap_ref(file, num)
     FILE *file;
     int num;
{
  int page;
  write_scrap_ref(file, num, TRUE, &page);
}
@| write_single_scrap_ref @}


@d Warn (only once) about needing to...
@{{
  if (!already_warned) {
    fprintf(stderr, "%s: you'll need to rerun nuweb after running tex\n",
	    command_name);
    already_warned = TRUE;
  }
}@}

@d Global variable dec...
@{extern int already_warned;
@| already_warned @}

@d Global variable def...
@{int already_warned = 0;
@}

@o scraps.c
@{typedef struct {
  Slab *scrap;
  Slab *prev;
  int index;
} Manager;
@| Manager @}



@o scraps.c
@{static void push(c, manager)
     char c;
     Manager *manager;
{
  Slab *scrap = manager->scrap;
  int index = manager->index;
  scrap->chars[index++] = c;
  if (index == SLAB_SIZE) {
    Slab *new = (Slab *) arena_getmem(sizeof(Slab));
    scrap->next = new;
    manager->scrap = new;
    index = 0;
  }
  manager->index = index;
}
@| push @}

@o scraps.c
@{static void pushs(s, manager)
     char *s;
     Manager *manager;
{
  while (*s)
    push(*s++, manager);
}
@| pushs @}


@o scraps.c
@{int collect_scrap()
{
  Manager writer;
  @<Create new scrap...@>
  @<Accumulate scrap and return \verb|scraps++|@>
}
@| collect_scrap @}

@d Create new scrap, managed by \verb|writer|
@{{
  Slab *scrap = (Slab *) arena_getmem(sizeof(Slab));
  if ((scraps & 255) == 0)
    SCRAP[scraps >> 8] = (ScrapEntry *) 
        arena_getmem(256 * sizeof(ScrapEntry));
  scrap_array(scraps).slab = scrap;
  scrap_array(scraps).file_name = save_string(source_name);
  scrap_array(scraps).file_line = source_line;
  scrap_array(scraps).page = -1;
  scrap_array(scraps).letter = 0;
  writer.scrap = scrap;
  writer.index = 0;
}@}


@d Accumulate scrap... @{{
  int codestate = 0;
  int c = source_get();
  while (1) {
    switch (c) {
      case EOF: fprintf(stderr, "%s: unexpect EOF in scrap (%s, %d)\n",
			command_name, scrap_array(scraps).file_name,
			scrap_array(scraps).file_line);
		exit(-1);
      case '@@': @<Handle at-sign during scrap accumulation@>
		break;
      default:  if (codestate<2) push(c, &writer);
		c = source_get();
		break;
    }
  }
}@}

@d Handle at-sign during scrap accumulation @{{
  c = source_get();
  switch (c) {
    case '@@': if (codestate<2) pushs("@@@@", &writer);
	      c = source_get();
	      break;
    case '|': if (codestate>0) {codestate=2; break;}
@{
              @<Collect user-specified index entries@>
@| Notice the fall through! 
@}
    case '}': if (codestate==0) {
                push('\0', &writer);
	        return scraps++;
              } else if (codestate==2) {
                codestate=0; c = source_get();
                break;
              }
    case '<': @<Handle macro invocation in scrap@>
	      break;
    case '{': if (codestate==0) codestate=1;
              c = source_get();
              break;
    default : fprintf(stderr, "%s: unexpected @@%c in scrap (%s, %d)\n",
		      command_name, c, source_name, source_line);
              c = source_get();
	      break;/*exit(-1);*/
  }
}@}

@d Collect user-specified index entries
@{{
  do {
    char new_name[100];
    char *p = new_name;
    do 
      c = source_get();
    while (isspace(c));
    if (c != '@@') {
      Name *name;
      do {
	*p++ = c;
	c = source_get();
      } while (c != '@@' && !isspace(c));
      *p = '\0';
      name = name_add(&user_names, new_name);
      if (!name->defs || name->defs->scrap != scraps) {
	Scrap_Node *def = (Scrap_Node *) 
            arena_getmem(sizeof(Scrap_Node));
	def->scrap = scraps;
	def->next = name->defs;
	name->defs = def;
      }
    }
  } while (c != '@@');
  c = source_get();
  if (c != '}') {
    fprintf(stderr, "%s: unexpected @@%c in scrap (%s, %d)\n",
	    command_name, c, source_name, source_line);
    exit(-1);
  }
}@}


@d Handle macro invocation in scrap
@{{
  Name *name = collect_scrap_name();
  @<Save macro name@>
  @<Add current scrap to \verb|name|'s uses@>
  c = source_get();
}@}


@d Save macro name
@{{
  char *s = name->spelling;
  int len = strlen(s) - 1;
  pushs("@@<", &writer);
  while (len > 0) {
    push(*s++, &writer);
    len--;
  }
  if (*s == ' ')
    pushs("...", &writer);
  else
    push(*s, &writer);
  pushs("@@>", &writer);
}@}


@d Add current scrap to...
@{{
  if (!name->uses || name->uses->scrap != scraps) {
    Scrap_Node *use = (Scrap_Node *) arena_getmem(sizeof(Scrap_Node));
    use->scrap = scraps;
    use->next = name->uses;
    name->uses = use;
  }
}@}


@o scraps.c
@{static char pop(manager)
     Manager *manager;
{
  Slab *scrap = manager->scrap;
  int index = manager->index;
  char c = scrap->chars[index++];
  if (index == SLAB_SIZE) {
    manager->prev = scrap;
    manager->scrap = scrap->next;
    index = 0;
  }
  manager->index = index;
  return c;
}
@| pop @}



@o scraps.c
@{static Name *pop_scrap_name(manager)
     Manager *manager;
{
  char name[100];
  char *p = name;
  int c = pop(manager);
  while (TRUE) {
    if (c == '@@')
      @<Check for end of scrap name and return@>
    else {
      *p++ = c;
      c = pop(manager);
    }
  }
}
@| pop_scrap_name @}


@d Check for end of scrap name...
@{{
  c = pop(manager);
  if (c == '@@') {
    *p++ = c;
    c = pop(manager);
  }
  else if (c == '>') {
    if (p - name > 3 && p[-1] == '.' && p[-2] == '.' && p[-3] == '.') {
      p[-3] = ' ';
      p -= 2;
    }
    *p = '\0';
    return prefix_add(&macro_names, name);
  }
  else {
    fprintf(stderr, "%s: found an internal problem (1)\n", command_name);
    exit(-1);
  }
}@}


@o scraps.c
@{int write_scraps(file, defs, global_indent, indent_chars,
		   debug_flag, tab_flag, indent_flag)
     FILE *file;
     Scrap_Node *defs;
     int global_indent;
     char *indent_chars;
     char debug_flag;
     char tab_flag;
     char indent_flag;
{
  int indent = 0;
  while (defs) {
    @<Copy \verb|defs->scrap| to \verb|file|@>
    defs = defs->next;
  }
  return indent + global_indent;
}
@| write_scraps @}


@d Copy \verb|defs->scrap...
@{{
  char c;
  Manager reader;
  int line_number = scrap_array(defs->scrap).file_line;
  @<Insert debugging information if required@>
  reader.scrap = scrap_array(defs->scrap).slab;
  reader.index = 0;
  c = pop(&reader);
  while (c) {
    switch (c) {
      case '@@':  @<Check for macro invocation in scrap@>
		 break;
      case '\n': putc(c, file);
		 line_number++;
		 @<Insert appropriate indentation@>
		 break;
      case '\t': @<Handle tab...@>
		 break;
      default:	 putc(c, file);
		 indent_chars[global_indent + indent] = ' ';
		 indent++;
		 break;
    }
    c = pop(&reader);
  }
}@}


@d Insert debugging information if required
@{if (debug_flag) {
  fprintf(file, "\n#line %d \"%s\"\n",
	  line_number, scrap_array(defs->scrap).file_name);
  @<Insert appropr...@>
}@}


@d Insert approp...
@{{
  if (indent_flag) {
    if (tab_flag)
      for (indent=0; indent<global_indent; indent++)
	putc(' ', file);
    else
      for (indent=0; indent<global_indent; indent++)
	putc(indent_chars[indent], file);
  }
  indent = 0;
}@}


@d Handle tab characters on output
@{{
  if (tab_flag)
    @<Expand tab...@>
  else {
    putc('\t', file);
    indent_chars[global_indent + indent] = '\t';
    indent++;
  }
}@}



@d Check for macro invocation...
@{{
  c = pop(&reader);
  switch (c) {
    case '@@': putc(c, file);
	      indent_chars[global_indent + indent] = ' ';
	      indent++;
	      break;
    case '<': @<Copy macro into \verb|file|@>
	      @<Insert debugging information if required@>
	      break;
    default:  /* ignore, since we should already have a warning */
	      break;
  }
}@}

{\bf Somewhere in here we should detect actual pramaters?}

@d Copy macro into...
@{{
  Name *name = pop_scrap_name(&reader);
  if (name->mark) {
    fprintf(stderr, "%s: recursive macro discovered involving <%s>\n",
	    command_name, name->spelling);
    exit(-1);
  }
  if (name->defs) {
    name->mark = TRUE;
    indent = write_scraps(file, name->defs, global_indent + indent,
			  indent_chars, debug_flag, tab_flag, indent_flag);
    indent -= global_indent;
    name->mark = FALSE;
  }
  else if (!doc_flag)
    fprintf(stderr, "%s: macro never defined <%s>\n",
	    command_name, name->spelling);
}@}


\subsubsection{Collecting Page Numbers}

@d Function...
@{extern void collect_numbers();
@}

@o scraps.c
@{void collect_numbers(aux_name)
     char *aux_name;
{
  if (number_flag) {
    int i;
    for (i=1; i<scraps; i++)
      scrap_array(i).page = i;
  }
  else {
    FILE *aux_file = fopen(aux_name, "r");
    already_warned = FALSE;
    if (aux_file) {
      char aux_line[500];
      while (fgets(aux_line, 500, aux_file)) {
        int scrap_number;
        int page_number;
        char dummy[50];
        if (3 == sscanf(aux_line, "\\newlabel{scrap%d}{%[^}]}{%d}",
			&scrap_number, dummy, &page_number)) {
	  if (scrap_number < scraps)
	    scrap_array(scrap_number).page = page_number;
	  else
	    @<Warn...@>
        }
      }
      fclose(aux_file);
      @<Add letters to scraps with duplicate page numbers@>
    }
  }
}
@| collect_numbers @}

@d Add letters to scraps with...
@{{
  int scrap;
  for (scrap=2; scrap<scraps; scrap++) {
    if (scrap_array(scrap-1).page == scrap_array(scrap).page) {
      if (!scrap_array(scrap-1).letter)
        scrap_array(scrap-1).letter = 'a';
      scrap_array(scrap).letter = scrap_array(scrap-1).letter + 1;
    }
  }
}@}


\subsection{Names} \define{names}

@d Type de...
@{typedef struct scrap_node {
  struct scrap_node *next;
  int scrap;
} Scrap_Node;
@| Scrap_Node @}


@d Type de...
@{typedef struct name {
  char *spelling;
  struct name *llink;
  struct name *rlink;
  Scrap_Node *defs;
  Scrap_Node *uses;
  int mark;
  char tab_flag;
  char indent_flag;
  char debug_flag;
} Name;
@| Name @}

@d Global variable dec...
@{extern Name *file_names;
extern Name *macro_names;
extern Name *user_names;
@| file_names macro_names user_names @}

@d Global variable def...
@{Name *file_names = NULL;
Name *macro_names = NULL;
Name *user_names = NULL;
@}

@d Function pro...
@{extern Name *collect_file_name();
extern Name *collect_macro_name();
extern Name *collect_scrap_name();
extern Name *name_add();
extern Name *prefix_add();
extern char *save_string();
extern void reverse_lists();
@}

@o names.c
@{enum { LESS, GREATER, EQUAL, PREFIX, EXTENSION };

static int compare(x, y)
     char *x;
     char *y;
{
  int len, result;
  int xl = strlen(x);
  int yl = strlen(y);
  int xp = x[xl - 1] == ' ';
  int yp = y[yl - 1] == ' ';
  if (xp) xl--;
  if (yp) yl--;
  len = xl < yl ? xl : yl;
  result = strncmp(x, y, len);
  if (result < 0) return GREATER;
  else if (result > 0) return LESS;
  else if (xl < yl) {
    if (xp) return EXTENSION;
    else return LESS;
  }
  else if (xl > yl) {
    if (yp) return PREFIX;
    else return GREATER;
  }
  else return EQUAL;
}
@| compare LESS GREATER EQUAL PREFIX EXTENSION @}


@o names.c
@{char *save_string(s)
     char *s;
{
  char *new = (char *) arena_getmem((strlen(s) + 1) * sizeof(char));
  strcpy(new, s);
  return new;
}
@| save_string @}

@o names.c
@{static int ambiguous_prefix();

Name *prefix_add(root, spelling)
     Name **root;
     char *spelling;
{
  Name *node = *root;
  while (node) {
    switch (compare(node->spelling, spelling)) {
    case GREATER:   root = &node->rlink;
		    break;
    case LESS:      root = &node->llink;
		    break;
    case EQUAL:     return node;
    case EXTENSION: node->spelling = save_string(spelling);
		    return node;
    case PREFIX:    @<Check for ambiguous prefix@>
		    return node;
    }
    node = *root;
  }
  @<Create new name entry@>
}
@| prefix_add @}

Since a very short prefix might match more than one macro name, I need
to check for other matches to avoid mistakes. Basically, I simply
continue the search down {\em both\/} branches of the tree.

@d Check for ambiguous prefix
@{{
  if (ambiguous_prefix(node->llink, spelling) ||
      ambiguous_prefix(node->rlink, spelling))
    fprintf(stderr,
	    "%s: ambiguous prefix @@<%s...@@> (%s, line %d)\n",
	    command_name, spelling, source_name, source_line);
}@}

@o names.c
@{static int ambiguous_prefix(node, spelling)
     Name *node;
     char *spelling;
{
  while (node) {
    switch (compare(node->spelling, spelling)) {
    case GREATER:   node = node->rlink;
		    break;
    case LESS:      node = node->llink;
		    break;
    case EQUAL:
    case EXTENSION:
    case PREFIX:    return TRUE;
    }
  }
  return FALSE;
}
@}

{\tt get\_defs} searches the tree of user identifiers for the
identifier {\tt spelling}, and returns a pointer to the list of
definitions.

@o names.c 
@{Scrap_Node *get_defs(spelling,node)
  char *spelling;
  Name *node;
{
  int res;
  if (!node) return NULL;
  res = robs_strcmp(spelling,node->spelling);
  if (res<0) return get_defs(spelling,node->llink);
  if (res>0) return get_defs(spelling,node->rlink);
  return node->defs;
}
@}

Rob Shillingsburg suggested that I organize the index of
user-specified identifiers more traditionally; that is, not relying on
strict {\small ASCII} comparisons via \verb|strcmp|. Ideally, we'd like
to see the index ordered like this:
\begin{quote}
aardvark
Adam
atom
Atomic
atoms
\end{quote}
The function \verb|robs_strcmp| implements the desired predicate.

@o names.c
@{static int robs_strcmp(x, y)
     char *x;
     char *y;
{
  char *xx = x;
  char *yy = y;
  int xc = toupper(*xx);
  int yc = toupper(*yy);
  while (xc == yc && xc) {
    xx++;
    yy++;
    xc = toupper(*xx);
    yc = toupper(*yy);
  }
  if (xc != yc) return xc - yc;
  xc = *x;
  yc = *y;
  while (xc == yc && xc) {
    x++;
    y++;
    xc = *x;
    yc = *y;
  }
  if (isupper(xc) && islower(yc))
    return xc * 2 - (toupper(yc) * 2 + 1);
  if (islower(xc) && isupper(yc))
    return toupper(xc) * 2 + 1 - yc * 2;
  return xc - yc;
}
@| robs_strcmp @}

@o names.c
@{Name *name_add(root, spelling)
     Name **root;
     char *spelling;
{
  Name *node = *root;
  while (node) {
    int result = robs_strcmp(node->spelling, spelling);
    if (result > 0)
      root = &node->llink;
    else if (result < 0)
      root = &node->rlink;
    else
      return node;
    node = *root;
  }
  @<Create new name entry@>
}
@| name_add @}


@d Create new name...
@{{
  node = (Name *) arena_getmem(sizeof(Name));
  node->spelling = save_string(spelling);
  node->mark = FALSE;
  node->llink = NULL;
  node->rlink = NULL;
  node->uses = NULL;
  node->defs = NULL;
  node->tab_flag = TRUE;
  node->indent_flag = TRUE;
  node->debug_flag = FALSE;
  *root = node;
  return node;
}@}


Name terminated by whitespace.  Also check for ``per-file'' flags. Keep
skipping white space until we reach scrap.  Once the scrap start is
recognized, we skip a single following newline, so that the macro body
can be formatted neatly.  This avoids blank lines in the output from
concatenated macros of the form:
\begin{verbatim}
@@o outputfile @@{
line 1
@@}
@@o outputfile @@{
line 2
@@}
\end{verbatim}
There is no blank line in \verb"outputfile" between \verb"line 1" and
\verb"line 2".

@o names.c
@{Name *collect_file_name()
{
  Name *new_name;
  char name[100];
  char *p = name;
  int start_line = source_line;
  int c = source_get();
  while (isspace(c))
    c = source_get();
  while (isgraph(c)) {
    *p++ = c;
    c = source_get();
  }
  if (p == name) {
    fprintf(stderr, "%s: expected file name (%s, %d)\n",
	    command_name, source_name, start_line);
    exit(-1);
  }
  *p = '\0';
  new_name = name_add(&file_names, name);
  @<Handle optional per-file flags@>
  if (c != '@@' || source_get() != '{') {
    fprintf(stderr, "%s: expected @@{ after file name (%s, %d)\n",
	    command_name, source_name, start_line);
    exit(-1);
  }
  /*if (source_peek == '\n') source_get();*/
  return new_name;
}
@| collect_file_name @}

@d Handle optional per-file flags
@{{
  while (1) {
    while (isspace(c))
      c = source_get();
    if (c == '-') {
      c = source_get();
      do {
	switch (c) {
	  case 't': new_name->tab_flag = FALSE;
		    break;
	  case 'd': new_name->debug_flag = TRUE;
		    break;
	  case 'i': new_name->indent_flag = FALSE;
		    break;
	  default : fprintf(stderr, "%s: unexpected per-file flag (%s, %d)\n",
			    command_name, source_name, source_line);
		    break;
	}
	c = source_get();
      } while (!isspace(c));
    }
    else break;
  }
}@}



Name terminated by \verb+\n+ or \verb+@@{+; but keep skipping until \verb+@@{+
@o names.c
@{Name *collect_macro_name()
{
  char name[100];
  char *p = name;
  int start_line = source_line;
  int c = source_get();
  while (isspace(c))
    c = source_get();
  while (c != EOF) {
    switch (c) {
      case '@@':  @<Check for terminating at-sequence and return name@>
		 break;
      case '\t':
      case ' ':  *p++ = ' ';
		 do
		   c = source_get();
		 while (c == ' ' || c == '\t');
		 break;
      case '\n': @<Skip until scrap begins, then return name@>
      default:	 *p++ = c;
		 c = source_get();
		 break;
    }
  }
  fprintf(stderr, "%s: expected macro name (%s, %d)\n",
	  command_name, source_name, start_line);
  exit(-1);
  return NULL;  /* unreachable return to avoid warnings on some compilers */
}
@| collect_macro_name @}


@d Check for terminating...
@{{
  c = source_get();
  switch (c) {
    case '@@': *p++ = c;
	      break;
    case '{': @<Cleanup and install name@>
    case '<': @<Handle a formal macro parameter definition@>
              break;
    default:  fprintf(stderr,
		      "%s: unexpected @@%c in macro name (%s, %d)\n",
		      command_name, c, source_name, start_line);
	      exit(-1);
  }
}@}


@d Cleanup and install name
@{{
  if (p > name && p[-1] == ' ')
    p--;
  if (p - name > 3 && p[-1] == '.' && p[-2] == '.' && p[-3] == '.') {
    p[-3] = ' ';
    p -= 2;
  }
  if (p == name || name[0] == ' ') {
    fprintf(stderr, "%s: empty scrap name (%s, %d)\n",
	    command_name, source_name, source_line);
    exit(-1);
  }
  *p = '\0';
  /*if (source_peek == '\n') source_get();*/
  return prefix_add(&macro_names, name);
}@}


@d Skip until scrap...
@{{
  do
    c = source_get();
  while (isspace(c));
  if (c != '@@' || source_get() != '{') {
    fprintf(stderr, "%s: expected @@{ after macro name (%s, %d)\n",
	    command_name, source_name, start_line);
    exit(-1);
  }
  @<Cleanup and install name@>
}@}


Terminated by \verb+@@>+
@o names.c
@{Name *collect_scrap_name()
{
  char name[100];
  char *p = name;
  int c = source_get();
  while (c == ' ' || c == '\t')
    c = source_get();
  while (c != EOF) {
    switch (c) {
      case '@@':  @<Look for end of scrap name and return@>
		 break;
      case '\t':
      case ' ':  *p++ = ' ';
		 do
		   c = source_get();
		 while (c == ' ' || c == '\t');
		 break;
      default:	 if (!isgraph(c)) {
		   fprintf(stderr,
		     "%s: unexpected character `%c' in macro name (%s, %d)\n",
		     command_name, c, source_name, source_line);
		   exit(-1);
		 }
		 *p++ = c;
		 c = source_get();
		 break;
    }
  }
  fprintf(stderr, "%s: unexpected end of file (%s, %d)\n",
	  command_name, source_name, source_line);
  exit(-1);
  return NULL;  /* unreachable return to avoid warnings on some compilers */
}
@| collect_scrap_name @}


@d Look for end of scrap name...
@{{
  c = source_get();
  switch (c) {
    case '@@': *p++ = c;
	      c = source_get();
	      break;
    case '>': @<Cleanup and install name@>
    default:  fprintf(stderr,
		      "%s: unexpected @@%c in macro name (%s, %d)\n",
		      command_name, c, source_name, source_line);
	      exit(-1);
  }
}@}


@o names.c
@{static Scrap_Node *reverse();	/* a forward declaration */

void reverse_lists(names)
     Name *names;
{
  while (names) {
    reverse_lists(names->llink);
    names->defs = reverse(names->defs);
    names->uses = reverse(names->uses);
    names = names->rlink;
  }
}
@| reverse_lists @}

Just for fun, here's a non-recursive version of the traditional list
reversal code. Note that it reverses the list in place; that is, it
does no new allocations.
@o names.c
@{static Scrap_Node *reverse(a)
     Scrap_Node *a;
{
  if (a) {
    Scrap_Node *b = a->next;
    a->next = NULL;
    while (b) {
      Scrap_Node *c = b->next;
      b->next = a;
      a = b;
      b = c;
    }
  }
  return a;
}
@| reverse @}


\subsection{Searching for Index Entries} \define{search}

Given the array of scraps and a set of index entries, we need to
search all the scraps for occurrences of each entry. The obvious
approach to this problem would be quite expensive for large documents;
however, there is an interesting  paper describing an efficient
solution~\cite{aho75}.


@o scraps.c
@{typedef struct name_node {
  struct name_node *next;
  Name *name;
} Name_Node;
@| Name_Node @}

@o scraps.c
@{typedef struct goto_node {
  Name_Node *output;		/* list of words ending in this state */
  struct move_node *moves;	/* list of possible moves */
  struct goto_node *fail;	/* and where to go when no move fits */
  struct goto_node *next;	/* next goto node with same depth */
} Goto_Node;
@| Goto_Node @}

@o scraps.c
@{typedef struct move_node {
  struct move_node *next;
  Goto_Node *state;
  char c;
} Move_Node;
@| Move_Node @}

@o scraps.c
@{static Goto_Node *root[128];
static int max_depth;
static Goto_Node **depths;
@| root max_depth depths @}


@o scraps.c
@{static Goto_Node *goto_lookup(c, g)
     char c;
     Goto_Node *g;
{
  Move_Node *m = g->moves;
  while (m && m->c != c)
    m = m->next;
  if (m)
    return m->state;
  else
    return NULL;
}
@| goto_lookup @}


\subsubsection{Building the Automata}


@d Function pro...
@{extern void search();
@}

@o scraps.c
@{static void build_gotos();
static int reject_match();

void search()
{
  int i;
  for (i=0; i<128; i++)
    root[i] = NULL;
  max_depth = 10;
  depths = (Goto_Node **) 
      arena_getmem(max_depth * sizeof(Goto_Node *));
  for (i=0; i<max_depth; i++)
    depths[i] = NULL;
  build_gotos(user_names);
  @<Build failure functions@>
  @<Search scraps@>
}
@| search @}



@o scraps.c
@{static void build_gotos(tree)
     Name *tree;
{
  while (tree) {
    @<Extend goto graph with \verb|tree->spelling|@>
    build_gotos(tree->rlink);
    tree = tree->llink;
  }
}
@| build_gotos @}

@d Extend goto...
@{{
  int depth = 2;
  char *p = tree->spelling;
  char c = *p++;
  Goto_Node *q = root[c];
  if (!q) {
    q = (Goto_Node *) arena_getmem(sizeof(Goto_Node));
    root[c] = q;
    q->moves = NULL;
    q->fail = NULL;
    q->moves = NULL;
    q->output = NULL;
    q->next = depths[1];
    depths[1] = q;
  }
  while (c = *p++) {
    Goto_Node *new = goto_lookup(c, q);
    if (!new) {
      Move_Node *new_move = 
        (Move_Node *) arena_getmem(sizeof(Move_Node));
      new = (Goto_Node *) arena_getmem(sizeof(Goto_Node));
      new->moves = NULL;
      new->fail = NULL;
      new->moves = NULL;
      new->output = NULL;
      new_move->state = new;
      new_move->c = c;
      new_move->next = q->moves;
      q->moves = new_move;
      if (depth == max_depth) {
	int i;
	Goto_Node **new_depths = (Goto_Node **) 
            arena_getmem(2*depth*sizeof(Goto_Node *));
	max_depth = 2 * depth;
	for (i=0; i<depth; i++)
	  new_depths[i] = depths[i];
	depths = new_depths;
	for (i=depth; i<max_depth; i++)
	  depths[i] = NULL;
      }
      new->next = depths[depth];
      depths[depth] = new;
    }
    q = new;
    depth++;
  }
  q->output = (Name_Node *) arena_getmem(sizeof(Name_Node));
  q->output->next = NULL;
  q->output->name = tree;
}@}


@d Build failure functions
@{{
  int depth;
  for (depth=1; depth<max_depth; depth++) {
    Goto_Node *r = depths[depth];
    while (r) {
      Move_Node *m = r->moves;
      while (m) {
	char a = m->c;
	Goto_Node *s = m->state;
	Goto_Node *state = r->fail;
	while (state && !goto_lookup(a, state))
	  state = state->fail;
	if (state)
	  s->fail = goto_lookup(a, state);
	else
	  s->fail = root[a];
	if (s->fail) {
	  Name_Node *p = s->fail->output;
	  while (p) {
	    Name_Node *q = (Name_Node *) arena_getmem(sizeof(Name_Node));
	    q->name = p->name;
	    q->next = s->output;
	    s->output = q;
	    p = p->next;
	  }
	}
	m = m->next;
      }
      r = r->next;
    }
  }
}@}


\subsubsection{Searching the Scraps}

@d Search scraps
@{{
  for (i=1; i<scraps; i++) {
    char c;
    Manager reader;
    Goto_Node *state = NULL;
    reader.prev = NULL;
    reader.scrap = scrap_array(i).slab;
    reader.index = 0;
    c = pop(&reader);
    while (c) {
      while (state && !goto_lookup(c, state))
	state = state->fail;
      if (state)
	state = goto_lookup(c, state);
      else
	state = root[c];
      c = pop(&reader);
      if (state && state->output) {
	Name_Node *p = state->output;
	do {
	  Name *name = p->name;
	  if (!reject_match(name, c, &reader) &&
	      (!name->uses || name->uses->scrap != i)) {
	    Scrap_Node *new_use =
		(Scrap_Node *) arena_getmem(sizeof(Scrap_Node));
	    new_use->scrap = i;
	    new_use->next = name->uses;
	    name->uses = new_use;
	  }
	  p = p->next;
	} while (p);
      }
    }
  }
}@}


\subsubsubsection{Rejecting Matches}

A problem with simple substring matching is that the string ``he''
would match longer strings like ``she'' and ``her.'' Norman Ramsey
suggested examining the characters occurring immediately before and
after a match and rejecting the match if it appears to be part of a
longer token. Of course, the concept of {\sl token\/} is
language-dependent, so we may be occasionally mistaken.
For the present, we'll consider the mechanism an experiment.

@o scraps.c
@{#define sym_char(c) (isalnum(c) || (c) == '_')

static int op_char(c)
     char c;
{
  switch (c) {
    case '!': case '@@': case '#': case '%': case '$': case '^': 
    case '&': case '*': case '-': case '+': case '=': case '/':
    case '|': case '~': case '<': case '>':
      return TRUE;
    default:
      return FALSE;
  }
}
@| sym_char op_char @}

@o scraps.c
@{static int reject_match(name, post, reader)
     Name *name;
     char post;
     Manager *reader;
{
  int len = strlen(name->spelling);
  char first = name->spelling[0];
  char last = name->spelling[len - 1];
  char prev = '\0';
  len = reader->index - len - 2;
  if (len >= 0)
    prev = reader->scrap->chars[len];
  else if (reader->prev)
    prev = reader->scrap->chars[SLAB_SIZE - len];
  if (sym_char(last) && sym_char(post)) return TRUE;
  if (sym_char(first) && sym_char(prev)) return TRUE;
  if (op_char(last) && op_char(post)) return TRUE;
  if (op_char(first) && op_char(prev)) return TRUE;
  return FALSE;
}
@| reject_match @}

\subsection{Macro Parameters}\define{macroparameters}

@d Handle a formal macro parameter definition @{
@}

\subsection{Memory Management} \define{memorymanagement}

I manage memory using a simple scheme inspired by Hanson's idea of
{\em arenas\/}~\cite{hanson90}.
Basically, I allocate all the storage required when processing a
source file (primarily for names and scraps) using calls to 
\verb|arena_getmem(n)|, where \verb|n| specifies the number of bytes to
be allocated. When the storage is no longer required, the entire arena
is freed with a single call to  \verb|arena_free()|. Both operations
are quite fast.
@d Function p...
@{extern void *arena_getmem();
extern void arena_free();
@}


@o arena.c
@{typedef struct chunk {
  struct chunk *next;
  char *limit;
  char *avail;
} Chunk;
@| Chunk @}


We define an empty chunk called \verb|first|. The variable \verb|arena| points
at the current chunk of memory; it's initially pointed at \verb|first|.
As soon as some storage is required, a ``real'' chunk of memory will
be allocated and attached to \verb|first->next|; storage will be
allocated from the new chunk (and later chunks if necessary).
@o arena.c
@{static Chunk first = { NULL, NULL, NULL };
static Chunk *arena = &first;
@| first arena @}


\subsubsection{Allocating Memory}

The routine \verb|arena_getmem(n)| returns a pointer to (at least) 
\verb|n| bytes of memory. Note that \verb|n| is rounded up to ensure
that returned pointers are always aligned.  We align to the nearest
8~byte segment, since that'll satisfy the more common 2-byte and
4-byte alignment restrictions too.

@o arena.c
@{void *arena_getmem(n)
     size_t n;
{
  char *q;
  char *p = arena->avail;
  n = (n + 7) & ~7;		/* ensuring alignment to 8 bytes */
  q = p + n;
  if (q <= arena->limit) {
    arena->avail = q;
    return p;
  }
  @<Find a new chunk of memory@>
}
@| arena_getmem @}


If the current chunk doesn't have adequate space (at least \verb|n|
bytes) we examine the rest of the list of chunks (starting at 
\verb|arena->next|) looking for a chunk with adequate space. If \verb|n|
is very large, we may not find it right away or we may not find a
suitable chunk at all.
@d Find a new chunk...
@{{
  Chunk *ap = arena;
  Chunk *np = ap->next;
  while (np) {
    char *v = sizeof(Chunk) + (char *) np;
    if (v + n <= np->limit) {
      np->avail = v + n;
      arena = np;
      return v;
    }
    ap = np;
    np = ap->next;
  }
  @<Allocate a new chunk of memory@>
}@}


If there isn't a suitable chunk of memory on the free list, then we
need to allocate a new one.
@d Allocate a new ch...
@{{
  size_t m = n + 10000;
  np = (Chunk *) malloc(m);
  np->limit = m + (char *) np;
  np->avail = n + sizeof(Chunk) + (char *) np;
  np->next = NULL;
  ap->next = np;
  arena = np;
  return sizeof(Chunk) + (char *) np;
}@}


\subsubsection{Freeing Memory}

To free all the memory in the arena, we need only point \verb|arena|
back to the first empty chunk.
@o arena.c
@{void arena_free()
{
  arena = &first;
}
@| arena_free @}

\section{The Additional {\TeX} Macros}

Since for plain {\TeX} output we do not have access to several of
{\LaTeX}'s facilities, we must cater for them.  These fall into three
main groups; verbatim, fonts and cross-referencing.  Other features of
{\LaTeX} can be added by other macro packages, such as the {\TeX} In
Practice ones, or my own (AJH) package, \verb"mytex".  Indeed, much of
the following is taken from the latter.

@o nutmacs.tex @{
@<{\tt nutmacs} verbatim@>
@<{\tt nutmacs} fonts@>
@<{\tt nutmacs} cross-referencing@>
@<{\tt nutmacs} parameters@>
@}

First verbatim.  This is essential for the code fragment typesetting.
The code is largely cribbed from the {\TeX}Book.  We have to nurgle
the definition of obeylines, to make sure that blank lines get out of
vmode, and hence do in fact make a blank line.
@d {\tt nutmacs} verbatim @{
\def\obeyspaces{\catcode`\ =\active}
{\obeyspaces\global\let =\ }
\def\mpar{\leavevmode\par}
{\catcode`\^^M=\active%
\gdef\obeylines{\catcode`\^^M=\active\let^^M=\mpar}%
\global\let^^M=\mpar}%
\def\uncatcodespecials{\def\do##1{\catcode`##1=12}\dospecials}
{ \catcode`\{=12 \catcode`\}=12 \catcode`\<=1 \catcode`\>=2%
  \catcode`\|=0 \catcode`\\=12%
  |obeylines|gdef|doverbatim^^M#1\end{verbatim}%
<#1|endgroup|endcommonpar|kern-|parskip|ignorespaces>>
\def\setupverbatim{\tt\relax
  \def\par{\leavevmode\endgraf}\parskip=0pt
  \obeylines\uncatcodespecials
  \obeyspaces}
\def\verb{\begingroup\setupverbatim\doverb}
\def\doverb#1{\def\next##1#1{##1\endgroup}\next}
\def\verbatim{\narrower\setupverbatim\doverbatim}%
@}

We also have to handle the two sizes of font required (plain {\TeX}
only has 10 point).  The smaller font is used for the
cross-referencing pointers.

@d {\tt nutmacs} fonts @{
\font\eightrm=cmr8
\font\eightbf=cmbx8
\font\eightit=cmti8
\font\eighttt=cmtt8
\font\eightsy=cmsy8
\font\eightmit=cmmi8
\font\tenrm=cmr10
\font\tenbf=cmbx10
\font\tenit=cmti10
\font\tentt=cmtt10
\font\tensf=cmss10
\font\tensy=cmsy10
\font\tenmit=cmmi10
% the small fonts
\def\nutsmall{%
  \def\rm{\eightrm}%
  \def\bf{\eightbf}%
  \def\it{\eightit}%
  \def\tt{\eighttt}%
  \textfont1\eightmit
  \textfont 2 \eightsy
  \rm
}
% the normal fonts
\def\nutnormal{%
  \def\rm{\tenrm}%
  \def\bf{\tenbf}%
  \def\it{\tenit}%
  \def\tt{\tentt}%
  \def\sf{\tensf}%
  \textfont 1 \tenmit
  \scriptfont 1 \eightmit
  \scriptfont 4 \eightit
  \textfont 2 \tensy
  \rm
}
\nutnormal\rm
\raggedbottom
\parskip6pt
\def\footnotesize{\nutsmall}
\newdimen\codemargin\codemargin=16pt
\newdimen\refsmargin\refsmargin=\codemargin
@}

The cross-referencing macros provide sufficient functionality to
create the \verb".aux" file in the {\TeX} processing phase.  This file
is used by nuweb to label the scraps.

The entries in this file mirror those generated by {\LaTeX} exactly,
thus avoiding having to rewrite another part of nuweb.  Since these
entries include a chapter/section number, the dummy macro
\verb"\nutsection" is invoked.  This macro may be redefined if you
want to add section numbering (as I have, elsewhere; but that is
beyond the scope of this document).

Note that \verb"\bye" is redefined.  This is so that the \verb".aux"
file can be properly closed.  The macro \verb"\nutend" is called.
This may be redefined, as long as it calls \verb"\nutlabend" to do the
actual closing.

@d {\tt nutmacs} cross-referencing @{
\catcode`\@@=11
\newwrite\nut@@auxout
\def\nutsection{}
\immediate\openout\nut@@auxout=\jobname.aux
\def\label#1{%
    \write\nut@@auxout{%
        \string\newlabel{#1}{{\nutsection}{\the\pageno}}%
    }%
}%
\def\nutlabend{%
    \immediate\closeout\nut@@auxout
}
\catcode`\@@=12
\def\nutend{\nutlabend}
\outer\def\bye{\par\vfill\supereject\nutend\end}
@}

At the moment, the only parameter is the side-by-side relative
spacing.  \verb"\codeside" is the width of code, \verb"\docoside" is
the width of documentation.  They should add to unity.

@d {\tt nutmacs} parameters @{
\def\codeside{0.6}
\def\docoside{0.4}
@}

\vfill\eject
\section{Indices} \define{indices}

Three sets of indices can be created automatically: an index of file
names, an index of macro names, and an index of user-specified
identifiers. An index entry includes the name of the entry, where it
was defined, and where it was referenced.

\subsection{Files}

@f

\subsection{Macros}

@m

\subsection{Identifiers}

Knuth prints his index of identifiers in a two-column format.
I could force this automatically by emitting the \verb|\twocolumn|
command; but this has the side effect of forcing a new page.
Therefore, it seems better to leave it this up to the user.

@u

\fi

\vfill\eject
\section{Bibliography}
\begin{bibliography}\def\newblock{}
\item{knuth84}Donald E. Knuth.  Literate Programming.  {\it The
Computer Journal}, 27(2):97-111, May 1984.
\item{aho75}Alfred~V. Aho and Margaret~J. Corasick.
\newblock Efficient string matching: An aid to bibliographic search.
\newblock {\em Communications of the ACM}, 18(6):333--340, June 1975.

\def\hp{\discretionary{}{}{}}
\item{drakos94}Nikos Drakos.
\newblock The {LaTeX2HTML} translator, January 1994.
\newblock Available from {\tt
  http://clb.\hp leeds\hp.\hp ac\hp.uk/nikos/tex2html/latex2html.tar}.

\item{hanson90}David~R. Hanson.
\newblock Fast allocation and deallocation of memory based on object lifetimes.
\newblock {\em Software -- Practice and Experience}, 20(1):5--12, January 1990.

\item{knuth84}Donald~E. Knuth.
\newblock Literate programming.
\newblock {\em The Computer Journal}, 27(2):97--111, May 1984.

\item{metafontprogram}Donald~E. Knuth.
\newblock {\em {{\small\sf METAFONT:}} The Program}.
\newblock Computers \& Typesetting. Addison-Wesley, 1986.

\item{texprogram}Donald~E. Knuth.
\newblock {\em {{\TeX}}: The Program}.
\newblock Computers \& Typesetting. Addison-Wesley, 1986.

\item{texbook}Donald~E. Knuth.
\newblock {\em The {{\TeX}}book}.
\newblock Computers \& Typesetting. Addison-Wesley, 1986.

\item{latex}Leslie Lamport.
\newblock {\em {{\LaTeX:}} A Document Preparation System}.
\newblock Addison-Wesley, 1986.

\item{levy90}Silvio Levy and Donald~E. Knuth.
\newblock {{\tt CWEB}} user manual: The {{\small CWEB}} system of structured
  documentation.
\newblock Technical Report {\small STAN}-{\small CS}-83-977, Stanford
  University, October 1990.
\newblock Available for anonymous ftp from {\tt labrea.stanford.edu} in
  directory {\tt pub/cweb}.

\item{noweb}Norman Ramsey,
``Literate programming simplified'',
{\it IEEE Software}, 11(5):97-105, September 1994.

\item{funnelweb}Ross~N. Williams,
``{FunnelWeb} user's manual'', May 1992.
Available for anonymous ftp from {\tt sirius.itd.adelaide.edu.au} in
  directory {\tt pub/funnelweb}.

\end{bibliography}
\vfill\eject

\bye
